@@ -6,110 +6,6 @@ using System.Threading;
 using FSpot.Widgets;
 
 namespace FSpot {
-	public class SepiaTone : ColorAdjustment {
-		public SepiaTone (Photo photo) : base (photo)
-		{
-		}
-
-		protected override Cms.Profile GenerateProfile ()
-		{
-			return Cms.Profile.CreateAbstract (nsteps,
-							   1.0,
-							   32.0,
-							   0.0,
-							   0.0,
-							   -100.0,
-							   null,
-							   ColorCIExyY.WhitePointFromTemperature (5000),
-							   ColorCIExyY.WhitePointFromTemperature (9934));
-		}
-	}
-
-	public class Desaturate : ColorAdjustment {
-		public Desaturate (Photo photo) : base (photo)
-		{
-		}
-
-		protected override Cms.Profile GenerateProfile ()
-		{
-			return Cms.Profile.CreateAbstract (nsteps,
-							   1.0,
-							   0.0,
-							   0.0,
-							   0.0,
-							   -100.0,
-							   null,
-							   ColorCIExyY.WhitePointFromTemperature (5000),
-							   ColorCIExyY.WhitePointFromTemperature (5000));
-	
-		}
-	}
-
-	public abstract class ColorAdjustment {
-		protected Photo photo;
-		protected Cms.Profile image_profile;
-		protected Cms.Profile destination_profile;
-		protected Cms.Profile adjustment_profile;
-		protected Gdk.Pixbuf image;
-		protected int nsteps = 20;
-		protected Cms.Intent intent = Cms.Intent.Perceptual;
-
-		public Gdk.Pixbuf Pixbuf {
-			get { return image; }
-			set { image = value; }
-		}
-
-		public ColorAdjustment (Photo photo)
-		{
-			this.photo = photo;
-		}
-		
-		public void SetDestination (Cms.Profile profile)
-		{
-			destination_profile = profile;
-		}
-
-		protected abstract Cms.Profile GenerateProfile ();
-
-		public void Adjust ()
-		{
-			bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
-			ImageFile img = ImageFile.Create (photo.DefaultVersionUri);
-
-			if (image == null)
-				image = img.Load ();
-			
-			if (image_profile == null)
-				image_profile = img.GetProfile ();
-			
-			if (image_profile == null)
-				image_profile = Cms.Profile.CreateStandardRgb ();
-
-			if (destination_profile == null)
-				destination_profile = image_profile;
-
-			Gdk.Pixbuf final = new Gdk.Pixbuf (Gdk.Colorspace.Rgb,
-							   false, 8,
-							   image.Width, 
-							   image.Height);
-			
-			Cms.Profile adjustment_profile = GenerateProfile ();
-			Cms.Profile [] list;
-			if (adjustment_profile != null)
-				list = new Cms.Profile [] { image_profile, adjustment_profile, destination_profile };
-			else
-				list = new Cms.Profile [] { image_profile, destination_profile };
-
-			Cms.Transform transform = new Cms.Transform (list,
-								     PixbufUtils.PixbufCmsFormat (image),
-								     PixbufUtils.PixbufCmsFormat (final),
-								     intent, 0x0000);
-			
-			PixbufUtils.ColorAdjust (image, final, transform);
-			photo.SaveVersion (final, create_version);
-		}
-	}
-
 	public class ColorDialog : GladeDialog {
 		protected static ColorDialog instance = null;
 		
@@ -245,9 +141,8 @@ namespace FSpot {
 		
 		public void HandleDestroyed (object sender, EventArgs arg)
 		{
-			view.Transform = null;
-			view.PhotoChanged -= HandlePhotoChanged;
-			view.QueueDraw ();
+			
+			Cancel ();
 #if USE_THREAD
 			expose_timeout.Stop ();
 #endif
@@ -284,7 +179,7 @@ namespace FSpot {
 
 			Photo photo = (Photo)view.Item.Current;
 			try {
-				bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
+				bool create_version = photo.DefaultVersion.IsProtected;
 				
 				Gdk.Pixbuf orig = view.CompletePixbuf ();
 				Gdk.Pixbuf final = new Gdk.Pixbuf (Gdk.Colorspace.Rgb,
@@ -311,6 +206,7 @@ namespace FSpot {
 				
 				photo.SaveVersion (final, create_version);
 				((PhotoQuery)view.Query).Commit (view.Item.Index);
+				final.Dispose ();
 			} catch (System.Exception e) {
 				string msg = Catalog.GetString ("Error saving adjusted photo");
 				string desc = String.Format (Catalog.GetString ("Received exception \"{0}\". Unable to save photo {1}"),
@@ -331,11 +227,11 @@ namespace FSpot {
 		
 		public void Cancel ()
 		{
+			this.Dialog.Destroy ();
 			view.Transform = null;
 			view.PhotoChanged -= HandlePhotoChanged;
 			view.QueueDraw ();
 			System.Console.WriteLine ("clearing window");
-			this.Dialog.Destroy ();
 			instance = null;
 		}
 		
@@ -360,21 +256,21 @@ namespace FSpot {
 				if (!view.Item.IsValid)
  					throw new Exception ("Invalid Image");
 				
-				FSpot.ImageFile img = FSpot.ImageFile.Create (((Photo)view.Item.Current).DefaultVersionUri);
-			
- 				try {
- 					image_profile = img.GetProfile ();
- 				} catch (System.Exception e) {
- 					image_profile = null;
- 					System.Console.WriteLine (e);
- 				}
+				using (FSpot.ImageFile img = FSpot.ImageFile.Create (((Photo)view.Item.Current).DefaultVersionUri)) {
+ 					try {
+ 						image_profile = img.GetProfile ();
+ 					} catch (System.Exception e) {
+ 						image_profile = null;
+ 						System.Console.WriteLine (e);
+ 					}
 
-				// FIXME fall back to rgb for now
-				if (image_profile == null)
-					image_profile = Cms.Profile.CreateStandardRgb ();
+					// FIXME fall back to rgb for now
+					if (image_profile == null)
+						image_profile = Cms.Profile.CreateStandardRgb ();
 				
-				AdjustedPixbuf = img.Load (256, 256);
-				ScaledPixbuf = AdjustedPixbuf.Copy ();			
+					AdjustedPixbuf = img.Load (256, 256);
+					ScaledPixbuf = AdjustedPixbuf.Copy ();			
+				}
 
 				if (AdjustedPixbuf.HasAlpha)
 					throw new Exception ("Unsupported Alpha Channel");
