@@ -181,7 +181,7 @@ namespace FSpot {
 		}
 	}
 
-	public class FileBrowsableItem : IBrowsableItem {
+	public class FileBrowsableItem : IBrowsableItem, IDisposable {
 		ImageFile img;
 		Uri uri;
 		bool attempted;
@@ -244,5 +244,11 @@ namespace FSpot {
 				return Path.GetFileName (Image.Uri.AbsolutePath);
 			}
 		}
+
+		public void Dispose ()
+		{
+			img.Dispose ();
+			GC.SuppressFinalize (this);
+		}
 	}
 }
