@@ -273,9 +273,12 @@ namespace FSpot {
 			fr.Connection.OnUploadProgress += HandleFlickrProgress;
 
 			System.Collections.ArrayList ids = new System.Collections.ArrayList ();
-			for (int index = 0; index < selection.Items.Length; index++) {
+			Photo [] photos = (Photo []) selection.Items;
+			Array.Sort (photos, new Photo.CompareDateName ());
+
+			for (int index = 0; index < photos.Length; index++) {
 				try {
-				 	IBrowsableItem photo = selection.Items [index];
+				 	IBrowsableItem photo = photos [index];
 					progress_dialog.Message = System.String.Format (
                                                 Catalog.GetString ("Uploading picture \"{0}\""), photo.Name);
 
@@ -310,8 +313,10 @@ namespace FSpot {
 					progress_dialog.ProgressText = Catalog.GetString ("Error");
 					System.Console.WriteLine (e);
 
-					if (progress_dialog.PerformRetrySkip ())
+					if (progress_dialog.PerformRetrySkip ()) {
 					 	index--;
+						photo_index--;
+					}
 				}
 			}
 
