@@ -30,7 +30,7 @@ using ICSharpCode.SharpZipLib.Zip;
 using ICSharpCode.SharpZipLib.GZip;
 
 namespace FSpot {
-	public class FolderExport : GladeDialog {
+	public class FolderExport : GladeDialog, FSpot.Extensions.IExporter {
 		IBrowsableCollection selection;
 		[Glade.Widget] Gtk.ScrolledWindow thumb_scrolledwindow;
 		[Glade.Widget] Gtk.Entry name_entry;
@@ -66,7 +66,9 @@ namespace FSpot {
 		FSpot.ThreadProgressDialog progress_dialog;
 		System.Threading.Thread command_thread;
 		
-		public FolderExport (IBrowsableCollection selection) : base ("folder_export_dialog")
+		public FolderExport () : base ("folder_export_dialog")
+		{}
+		public void Run (IBrowsableCollection selection)
 		{
 			/*
 			Gnome.Vfs.ModuleCallbackFullAuthentication auth = new Gnome.Vfs.ModuleCallbackFullAuthentication ();
@@ -108,8 +110,14 @@ namespace FSpot {
 
 			uri_chooser = new Gtk.FileChooserButton (Catalog.GetString ("Select Export Folder"),
 								 Gtk.FileChooserAction.SelectFolder);
-			uri_chooser.SetFilename (uri_path);
+			
 			uri_chooser.LocalOnly = false;
+
+			if (Preferences.Get (Preferences.EXPORT_FOLDER_URI) != null && Preferences.Get (Preferences.EXPORT_FOLDER_URI) as string != String.Empty)
+				uri_chooser.SetUri (Preferences.Get (Preferences.EXPORT_FOLDER_URI) as string);
+			else
+				uri_chooser.SetFilename (uri_path);
+
 			chooser_hbox.PackStart (uri_chooser);
 
 			Dialog.ShowAll ();
@@ -146,7 +154,7 @@ namespace FSpot {
 					gallery_path = Gnome.Vfs.Uri.GetLocalPathFromUri (dest.ToString ());
 
 				progress_dialog.Message = Catalog.GetString ("Building Gallery");
-				progress_dialog.Fraction = 1.0;
+				progress_dialog.Fraction = 0.0;
 
 				FolderGallery gallery;
 				if (static_radio.Active) {
@@ -171,19 +179,48 @@ namespace FSpot {
 
 				gallery.Description = description;
 
-				gallery.Generate ();
+				gallery.GenerateLayout ();
+				Filters.FilterSet filter_set = new Filters.FilterSet ();
+				if (scale)
+					filter_set.Add (new Filters.ResizeFilter ((uint) size));
+				else if (rotate)
+					filter_set.Add (new Filters.OrientationFilter ());
+				filter_set.Add (new Filters.ChmodFilter ());
+				filter_set.Add (new Filters.UniqueNameFilter (gallery_path));
+
+				for (int photo_index = 0; photo_index < selection.Count; photo_index++)
+				{
+					try {	
+						progress_dialog.Message = System.String.Format (Catalog.GetString ("Uploading picture \"{0}\""), selection[photo_index].Name);
+						progress_dialog.Fraction = photo_index / (double) selection.Count;
+						gallery.ProcessImage (photo_index, filter_set);
+						progress_dialog.ProgressText = System.String.Format (Catalog.GetString ("{0} of {1}"), (photo_index + 1), selection.Count);
+					}
+					catch (Exception e) {
+						progress_dialog.Message = String.Format (Catalog.GetString ("Error uploading picture \"{0}\" to Gallery:{2}{1}"), 
+							selection[photo_index].Name, e.Message, Environment.NewLine);
+						progress_dialog.ProgressText = Catalog.GetString ("Error");
+
+						if (progress_dialog.PerformRetrySkip ())
+							photo_index--;
+					}
+		
+				}
+
+				//create the zip tarballs for original
+				if (gallery is OriginalGallery && (bool)Preferences.Get(Preferences.EXPORT_FOLDER_INCLUDE_TARBALLS))
+					(gallery as OriginalGallery).CreateZip ();
 
 				// we've created the structure, now if the destination was local we are done
 				// otherwise we xfer 
 				if (!dest.IsLocal) {
 					Console.WriteLine(target);
-					Gnome.Vfs.XferProgressCallback cb = new Gnome.Vfs.XferProgressCallback (Progress);
 					System.Console.WriteLine ("Xfering {0} to {1}", source.ToString (), target.ToString ());
 					result = Gnome.Vfs.Xfer.XferUri (source, target, 
 									 Gnome.Vfs.XferOptions.Default, 
 									 Gnome.Vfs.XferErrorMode.Abort, 
 									 Gnome.Vfs.XferOverwriteMode.Replace, 
-									 cb);
+									 Progress);
 				}
 
 				if (result == Gnome.Vfs.Result.Ok) {
@@ -208,8 +245,9 @@ namespace FSpot {
 				Preferences.Set (Preferences.EXPORT_FOLDER_OPEN, open);
 				Preferences.Set (Preferences.EXPORT_FOLDER_ROTATE, rotate);
 				Preferences.Set (Preferences.EXPORT_FOLDER_METHOD, static_radio.Active ? "static" : original_radio.Active ? "original" : "folder" );
-
+				Preferences.Set (Preferences.EXPORT_FOLDER_URI, uri_chooser.Uri);
 			} catch (System.Exception e) {
+				// Console.WriteLine (e);
 				progress_dialog.Message = e.ToString ();
 				progress_dialog.ProgressText = Catalog.GetString ("Error Transferring");
 			} finally {
@@ -352,8 +390,8 @@ namespace FSpot {
 
 		protected ScaleRequest [] requests;
 		
-		protected string [] pixbuf_keys = {"quality"};
-		protected string [] pixbuf_values = {"95"};
+		protected string [] pixbuf_keys = { "quality", null };
+		protected string [] pixbuf_values = { "95", null };
 
 		protected struct ScaleRequest {
 			public string Name;
@@ -366,14 +404,14 @@ namespace FSpot {
 
 			public ScaleRequest (string name, int width, int height, bool skip, bool exif)
 			{
-				this.Name = name != null ? name : "";
+				this.Name = name != null ? name : String.Empty;
 				this.Width = width;
 				this.Height = height;
 				this.Skip = skip;
 				this.CopyExif = exif;
 			}
 
-			public static ScaleRequest Default = new ScaleRequest ("", 0, 0, false);
+			public static ScaleRequest Default = new ScaleRequest (String.Empty, 0, 0, false);
 
 			public bool AvoidScale (int size) { 
 				return (size < this.Width && size < this.Height && this.Skip);
@@ -388,14 +426,10 @@ namespace FSpot {
 			this.requests = new ScaleRequest [] { ScaleRequest.Default };
 		}
 
-		public virtual void Generate ()
+		public virtual void GenerateLayout ()
 		{
 			MakeDir (gallery_path);
 
-			for (int i = 0; i < collection.Count; i++)
-			{
-				ProcessImage (i);
-			}
 		}
 
 		protected virtual string ImageName (int image_num)
@@ -403,7 +437,7 @@ namespace FSpot {
 			return System.IO.Path.GetFileName(FileImportBackend.UniqueName(gallery_path, System.IO.Path.GetFileName (collection [image_num].DefaultVersionUri.LocalPath))); 
 		}
 
-		public void ProcessImage (int image_num)
+		public void ProcessImage (int image_num, Filters.FilterSet filter_set)
 		{
 			IBrowsableItem photo = collection [image_num];
 			string photo_path = photo.DefaultVersionUri.LocalPath;
@@ -415,47 +449,48 @@ namespace FSpot {
 			MakeDir (SubdirPath (req.Name));
 			path = SubdirPath (req.Name, ImageName (image_num));
 			
-			if (scale) {
-				PixbufUtils.Resize (photo_path, path, size, true); 	
-			} else if (rotate && (new Filters.OrientationFilter ().Convert (photo_path, path))) {
-				// do nothing, all is well the filter did the copying
-			} else {
-				File.Copy (photo_path, path, true);
-			}
-
-			Gdk.Pixbuf img = null;
-			Gdk.Pixbuf scaled = null;
-
-			Exif.ExifData data = new Exif.ExifData (photo_path);
-			if (data != null && data.Handle.Handle == System.IntPtr.Zero)
-				data = null;
-
-			for (int i = 1; i < requests.Length; i++) {
-				req = requests [i];
-				if (scale && req.AvoidScale (size))
-					continue;
-
-				if (img == null) 
-					scaled = PixbufUtils.LoadAtMaxSize (photo_path, req.Width, req.Height);
+			using (Filters.FilterRequest request = new Filters.FilterRequest (photo.DefaultVersionUri)) {
+				filter_set.Convert (request);
+				if (request.Current.LocalPath == path)
+					request.Preserve(request.Current);
 				else
-					scaled = PixbufUtils.ScaleToMaxSize (img, req.Width, req.Height, false);
-				
-				MakeDir (SubdirPath (req.Name));
-				path = SubdirPath (req.Name, ImageName (image_num));
+					File.Copy (request.Current.LocalPath, path, true); 
 
-				if (req.CopyExif && data != null) {
-					PixbufUtils.SaveJpeg (scaled, path, 90, data);
-				} else 
-					scaled.Savev (path, "jpeg", pixbuf_keys, pixbuf_values);
+				if (photo != null && photo is Photo && Core.Database != null) {
+					Core.Database.Exports.Create ((photo as Photo).Id, (photo as Photo).DefaultVersionId,
+								      ExportStore.FolderExportType,
+								      // FIXME this is wrong, the final path is the one
+								      // after the Xfer.
+								      UriList.PathToFileUriEscaped (path).ToString ());
+				}
+
+				using (Exif.ExifData data = new Exif.ExifData (photo_path)) {
+					for (int i = 1; i < requests.Length; i++) {
+						
+						req = requests [i];
+						if (scale && req.AvoidScale (size))
+							continue;
 				
-				if (img != null)
-					img.Dispose ();
+						Filters.FilterSet req_set = new Filters.FilterSet ();
+						req_set.Add (new Filters.ResizeFilter ((uint)Math.Max (req.Width, req.Height)));
+						if ((bool)Preferences.Get (Preferences.EXPORT_FOLDER_SHARPEN)) {
+							if (req.Name == "lq")
+								req_set.Add (new Filters.SharpFilter (0.1, 2, 4));
+							if (req.Name == "thumbs")
+								req_set.Add (new Filters.SharpFilter (0.1, 2, 5));
+						}
+						using (Filters.FilterRequest tmp_req = new Filters.FilterRequest (photo.DefaultVersionUri)) {
+							req_set.Convert (tmp_req);
+							MakeDir (SubdirPath (req.Name));
+							path = SubdirPath (req.Name, ImageName (image_num));
+							System.IO.File.Copy (tmp_req.Current.LocalPath, path, true);
+						}
+						
+					}
+				}
 			}
-
-			if (scaled != null)
-				scaled.Dispose ();
 		}
-
+		
 		protected string MakeDir (string path)
 		{
 			try {
@@ -552,11 +587,10 @@ namespace FSpot {
 							 new ScaleRequest ("thumbs", 120, 120, false) };
 		}
 
-		public override void Generate ()
+		public override void GenerateLayout ()
 		{
-			base.Generate ();
+			base.GenerateLayout ();
 			MakeDir (SubdirPath ("comments"));
-			MakeDir (SubdirPath ("zip"));
 			CreateHtaccess();
 			CreateInfo();
 			SetTime ();
@@ -579,7 +613,15 @@ namespace FSpot {
 					CreateComments (collection [i].DefaultVersionUri.LocalPath, i);
 
 				Directory.SetLastWriteTimeUtc(gallery_path, collection [0].Time);
+			} catch (System.Exception e) {
+				System.Console.WriteLine (e.ToString ());
+			} 
+		}
 
+		internal void CreateZip () 
+		{
+			MakeDir (SubdirPath ("zip"));
+			try {
 				if (System.IO.Directory.Exists (SubdirPath ("mq")))
 				    CreateZipFile("mq");
 
@@ -595,7 +637,7 @@ namespace FSpot {
 		{
 			StreamWriter comment = File.CreateText(SubdirPath  ("comments", photo_index + 1 + ".txt"));
 			comment.Write("<span>photo " + (photo_index + 1) + "</span> ");
-			comment.Write (collection [photo_index].Description + "\n");
+			comment.Write (collection [photo_index].Description + Environment.NewLine);
 			comment.Close();
 		}
 
@@ -642,7 +684,7 @@ namespace FSpot {
 		private void CreateHtaccess()
 		{
 			StreamWriter htaccess = File.CreateText(Path.Combine (gallery_path,".htaccess"));
-			htaccess.Write("<Files info.txt>\n\tdeny from all\n</Files>\n");
+			htaccess.Write("<Files info.txt>" + Environment.NewLine + "\tdeny from all" + Environment.NewLine+ "</Files>" + Environment.NewLine);
 			htaccess.Close();
 		}
 
@@ -664,8 +706,8 @@ namespace FSpot {
 		string altstylesheet = "f-spot-simple-white.css";
 		string javascript = "f-spot.js";
 
-        static string light = Catalog.GetString("Light");
-        static string dark = Catalog.GetString("Dark");
+		static string light = Catalog.GetString("Light");
+		static string dark = Catalog.GetString("Dark");
 		
 		public HtmlGallery (IBrowsableCollection selection, string path, string name) : base (selection, path, name) 
 		{ 
@@ -679,12 +721,12 @@ namespace FSpot {
 			return String.Format ("img-{0}.jpg", photo_index + 1);
 		}
 
-		public override void Generate ()
+		public override void GenerateLayout ()
 		{
 			if (collection.Count == 0)
 				return;
 			
-			base.Generate ();
+			base.GenerateLayout ();
 			
 			IBrowsableItem [] photos = collection.Items;
 			
@@ -697,39 +739,42 @@ namespace FSpot {
 			
 			MakeDir (SubdirPath ("style"));
 			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetCallingAssembly ();
-			System.IO.Stream s = assembly.GetManifestResourceStream (stylesheet);
-			System.IO.Stream fs = System.IO.File.Open (SubdirPath ("style", stylesheet), System.IO.FileMode.Create);
+			using (Stream s = assembly.GetManifestResourceStream (stylesheet)) {
+				using (Stream fs = System.IO.File.Open (SubdirPath ("style", stylesheet), System.IO.FileMode.Create)) {
 
-			byte [] buffer = new byte [8192];
-			int n;
-			while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
-				fs.Write (buffer, 0,  n);						    
-			
-			fs.Close ();
+					byte [] buffer = new byte [8192];
+					int n;
+					while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
+						fs.Write (buffer, 0,  n);						    
+					
+				}
+			}
 			/* quick and stupid solution
 			   this should have been iterated over an array of stylesheets, really
 			*/
-			s = assembly.GetManifestResourceStream (altstylesheet);
-			fs = System.IO.File.Open (SubdirPath ("style", altstylesheet), System.IO.FileMode.Create);
+			using (Stream s = assembly.GetManifestResourceStream (altstylesheet)) {
+				using (Stream fs = System.IO.File.Open (SubdirPath ("style", altstylesheet), System.IO.FileMode.Create)) {
+					
+					byte [] buffer = new byte [8192];
+					int n = 0;
+					while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
+						fs.Write (buffer, 0,  n);						    
+					
+				}
+			}
 
-			buffer = new byte [8192];
-			n = 0;
-			while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
-				fs.Write (buffer, 0,  n);						    
-			
-			fs.Close ();
 			/* Javascript for persistant style change */
 			MakeDir (SubdirPath ("script"));
-			s = assembly.GetManifestResourceStream (javascript);
-			fs = System.IO.File.Open (SubdirPath ("script", javascript), System.IO.FileMode.Create);
+			using (Stream s = assembly.GetManifestResourceStream (javascript)) {
+				using (Stream fs = System.IO.File.Open (SubdirPath ("script", javascript), System.IO.FileMode.Create)) {
 
-			buffer = new byte [8192];
-			n = 0;
-			while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
-				fs.Write (buffer, 0,  n);						    
-			
-			fs.Close ();
-	
+					byte [] buffer = new byte [8192];
+					int n = 0;
+					while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
+						fs.Write (buffer, 0,  n);						    
+					
+				}
+			}
 		}
 		
 		public int PageCount {
@@ -864,7 +909,7 @@ namespace FSpot {
 			writer.Write ("<span class=\"style_toggle\">"); 
 			writer.Write ("<a href=\"javascript:toggle_stylebox()\">");
 			writer.Write ("<span id=\"showlink\">" + Catalog.GetString ("Show Styles") + "</span><span id=\"hidelink\" ");
-			writer.Write ("style=\"display:none;\">" + Catalog.GetString ("Hide Styles") + "</span></a></span>\n");
+			writer.Write ("style=\"display:none;\">" + Catalog.GetString ("Hide Styles") + "</span></a></span>" + Environment.NewLine);
 			writer.RenderEndTag (); //div toggle
 			writer.RenderEndTag (); //div styleboxcontainer
 			writer.RenderEndTag (); //container1	
@@ -903,15 +948,15 @@ namespace FSpot {
 
 			writer.Write ("<link type=\"text/css\" rel=\"stylesheet\" href=\"");
 			writer.Write (String.Format ("{0}", "style/" + stylesheet));
-			writer.Write ("\" title=\"" + dark + "\" media=\"screen\" />\n");
+			writer.Write ("\" title=\"" + dark + "\" media=\"screen\" />" + Environment.NewLine);
 
 			writer.Write ("<link type=\"text/css\" rel=\"prefetch ") ;
 			writer.Write ("alternate stylesheet\" href=\"");
 			writer.Write (String.Format ("{0}", "style/" + altstylesheet));
-			writer.Write ("\" title=\"" + light + "\" media=\"screen\" />\n");
+			writer.Write ("\" title=\"" + light + "\" media=\"screen\" />" + Environment.NewLine);
 
 			writer.Write ("<script src=\"script/" + javascript + "\"");
-			writer.Write (" type=\"text/javascript\"></script>\n");
+			writer.Write (" type=\"text/javascript\"></script>" + Environment.NewLine);
 
 			writer.RenderEndTag ();
 		}
@@ -1037,7 +1082,7 @@ namespace FSpot {
 			writer.Write ("<span class=\"style_toggle\">"); 
 			writer.Write ("<a href=\"javascript:toggle_stylebox()\">");
 			writer.Write ("<span id=\"showlink\">" + Catalog.GetString("Show Styles") + "</span><span id=\"hidelink\" ");
-			writer.Write ("style=\"display:none;\">" + Catalog.GetString("Hide Styles") + "</span></a></span>\n");
+			writer.Write ("style=\"display:none;\">" + Catalog.GetString("Hide Styles") + "</span></a></span>" + Environment.NewLine);
 			writer.RenderEndTag (); //div toggle
 			writer.RenderEndTag (); //div styleboxcontainer
 			writer.RenderEndTag (); //container1
