@@ -2,6 +2,12 @@
 using FSpot;
 using SemWeb;
 using System;
+using System.IO;
+using System.Collections.Generic;
+
+#if ENABLE_NUNIT
+using NUnit.Framework;
+#endif
 
 namespace FSpot.Tiff {
 
@@ -26,6 +32,40 @@ namespace FSpot.Tiff {
 		TileIndex                       = 33467
 	}
 
+	public enum TagGPS : ushort {
+		GPSVersionID                    = 0x0000,
+		GPSLatitudeRef                  = 0x0001,
+		GPSLatitude                     = 0x0002,
+			GPSLongitudeRef                    = 0x0003,
+			GPSLongitue                        = 0x0004,
+			GPSAltitudeRef                     = 0x0005,
+			GPSAltitude                        = 0x0006,
+			GPSTimeStamp                       = 0x0007,
+			GPSSatellites                      = 0x0008,
+			GPSStatus                          = 0x0009,
+			GPSMeasureMode                     = 0x000a,
+			GPSDOP                             = 0x000b,
+			GPSSpeedRef                        = 0x000c,
+			GPSSpeed                           = 0x000d,
+			GPSTrackRef                        = 0x000e,
+			GPSTrack                           = 0x000f,
+			GPSImgDirectionRef                 = 0x0010,
+			GPSImgDirection                    = 0x0011,
+			GPSMapDatum                        = 0x0012,
+			GPSDestLatitudeRef                 = 0x0013,
+			GPSDestLatitude                    = 0x0014,
+			GPSDestLongitudeRef                = 0x0015,
+			GPSDestLongitude                   = 0x0016,
+			GPSDestBearingRef                  = 0x0017,
+			GPSDestBearing                     = 0x0018,
+			GPSDestDistanceRef                 = 0x0019,
+			GPSDestDistance                    = 0x001a,
+			GPSProcessingMethod                = 0x001b,
+			GPSAreaInformation                 = 0x001c,
+			GPSDateStamp                       = 0x001d,
+			GPSDifferential                    = 0x001e
+	}
+
 	public enum TagId : ushort {
 		InteroperabilityIndex		= 0x0001,
 		InteroperabilityVersion	        = 0x0002,
@@ -323,7 +363,7 @@ namespace FSpot.Tiff {
 				this.Numerator = (uint) (tmp * 100000);
 				this.Denominator = 100000;
 			} else
-				throw new System.Exception ("unable to parse rational value");
+				throw new ParseException ("unable to parse rational value");
 		}
 
 		public override string ToString ()
@@ -359,7 +399,7 @@ namespace FSpot.Tiff {
 		{
 			if (raw_data.Length == 8 || raw_data.Length == 0) { 
 				Charset = null;
-				Value = "";
+				Value = String.Empty;
 				return;
 			} else if (raw_data.Length < 8) {
 				throw new Exception ("Invalid UserComment value, no charset found");
@@ -392,7 +432,7 @@ namespace FSpot.Tiff {
 				break;
 			default:
 				enc = null;
-				throw new System.Exception (System.String.Format ("Invalid charset name: {0}", charset));
+				throw new ParseException (System.String.Format ("Invalid charset name: {0}", charset));
 			}
 
 			Charset = charset;
@@ -591,7 +631,6 @@ namespace FSpot.Tiff {
 		Custom = 1
 	}
 
-
 	public enum SceneType {
 		DirectlyPhotographed = 1
 	}
@@ -805,9 +844,12 @@ namespace FSpot.Tiff {
 		{
 			byte [] tmp = new byte [4];
 
-		        if (stream.Read (tmp, 0, tmp.Length) < 4)
+		        if (stream.Read (tmp, 0, tmp.Length) < 4) {
+#if DEBUG_LOADER
+				System.Console.WriteLine ("short read XXXXXXXXXXXXXXXXXXXXXXx");
+#endif
 				throw new ShortReadException ();
-
+			}
 			return BitConverter.ToUInt32 (tmp, 0, endian == Endian.Little);
 		}
 
@@ -815,8 +857,12 @@ namespace FSpot.Tiff {
 		{
 			byte [] tmp = new byte [2];
 
-		        if (stream.Read (tmp, 0, tmp.Length) < 2)
+		        if (stream.Read (tmp, 0, tmp.Length) < 2) {
+#if DEBUG_LOADER
+				System.Console.WriteLine ("Short read");
+#endif
 				throw new ShortReadException ();
+			}
 
 			return BitConverter.ToUInt16 (tmp, 0, endian == Endian.Little);
 		}
@@ -934,7 +980,7 @@ namespace FSpot.Tiff {
 					MetadataStore.AddLiteral (sink, "xmp:ModifyDate", 
 								  e.ValueAsDate.ToString ("yyyy-MM-ddThh:mm:ss"));
 					} catch (System.Exception ex) {
-						System.Console.WriteLine (String.Format ("error parsing {0}\n{1}", e.ValueAsString[0], ex));
+						System.Console.WriteLine (String.Format ("error parsing {0}{2}{1}", e.ValueAsString[0], ex, Environment.NewLine));
 					}
 
 					break;
@@ -946,7 +992,7 @@ namespace FSpot.Tiff {
 						MetadataStore.AddLiteral (sink, "exif:" + e.Id.ToString (), 
 									  e.ValueAsDate.ToString ("yyyy-MM-ddThh:mm:ss"));
 					} catch (System.Exception ex) {
-						System.Console.WriteLine (String.Format ("error parsing {0}\n{1}", e.ValueAsString[0], ex));
+						System.Console.WriteLine (String.Format ("error parsing {0}{2}{1}", e.ValueAsString[0], ex, Environment.NewLine));
 					}
 					break;
 					//case TagId.SpatialFrequencyResponse
@@ -1035,11 +1081,35 @@ namespace FSpot.Tiff {
 				case TagId.ImageLength:
 				case TagId.Model:
 				case TagId.Make:
-					MetadataStore.AddLiteral (sink, "tiff:" + e.Id.ToString (), e.ValueAsString [0]);
+					try {
+						MetadataStore.AddLiteral (sink, "tiff:" + e.Id.ToString (), e.ValueAsString [0]);
+					} catch (System.Exception ex) {
+						System.Console.WriteLine (String.Format ("error parsing {0}{2}{1}", e.Id, ex, Environment.NewLine));
+					}
 					break;
 				}
 			}
 		}
+
+		public void Save (System.IO.Stream out_stream)
+		{
+			OrderedWriter writer = new OrderedWriter (out_stream, endian == Endian.Little);
+			
+			/* Header */
+			if (endian == Endian.Little) {
+				writer.Write ((byte)'I');
+				writer.Write ((byte)'I');
+			} else {
+				writer.Write ((byte)'M');
+				writer.Write ((byte)'M');
+			}
+			
+			writer.Write ((ushort)42);
+			
+			/* First IFD */
+			Directory.Save (writer, 8);
+		}
+
 		public void Dump (string name)
 		{
 			ImageDirectory ifd = Directory;
@@ -1053,7 +1123,7 @@ namespace FSpot.Tiff {
 	public class ImageDirectory {
 		protected Endian endian;
 		protected ushort num_entries;
-		protected System.Collections.ArrayList entries;
+		protected List<DirectoryEntry> entries;
 		protected uint orig_position;
 
 		protected uint next_directory_offset;
@@ -1069,6 +1139,30 @@ namespace FSpot.Tiff {
 			Load (stream);
 		}
 		
+		public uint Save (OrderedWriter writer, uint position)
+		{
+			writer.Write (position);
+			writer.Stream.Position = position;
+
+			writer.Write ((ushort)entries.Count);
+
+			position += 2;
+			uint  value_position = (uint) (position + 12 * entries.Count + 4);
+
+			for (int i = 0; i < entries.Count; i++) {
+				writer.Stream.Position = position + (12 * i);
+				value_position = (uint)Entries[i].Save (writer, value_position);
+			}
+							
+			writer.Stream.Position = position + (12 * entries.Count);
+			if (next_directory != null)
+				value_position = next_directory.Save (writer, value_position);
+			else 
+				writer.Write ((uint) 0);
+
+			return value_position;
+		}
+
 		protected void Load (System.IO.Stream stream)
 		{
 			ReadHeader (stream);			
@@ -1091,15 +1185,20 @@ namespace FSpot.Tiff {
 #if DEBUG_LOADER
 			System.Console.WriteLine ("reading {0} entries", num_entries);
 #endif			
-			entries = new System.Collections.ArrayList (num_entries);
+			entries = new List<DirectoryEntry> (num_entries);
 			int entry_length = num_entries * 12;
 			byte [] content = new byte [entry_length];
 			
-			if (stream.Read (content, 0, content.Length) < content.Length)
+			if (stream.Read (content, 0, content.Length) < content.Length) {
+#if DEBUG_LOADER
+				System.Console.WriteLine ("short read XXXXXXXXXXXXXXXXXXXXXXx");
+#endif
 				throw new ShortReadException ();
-			
+			}
+
+
 			for (int pos = 0; pos < entry_length; pos += 12) {
-				DirectoryEntry entry = EntryFactory.CreateEntry (this, content, pos, this.endian);
+				DirectoryEntry entry = CreateEntry (this, content, pos, this.endian);
 				entries.Add (entry);		
 #if DEBUG_LOADER
 				System.Console.WriteLine ("Added Entry {0} {1} - {2} * {3}", entry.Id.ToString (), entry.Id.ToString ("x"), entry.Type, entry.Count);
@@ -1125,7 +1224,8 @@ namespace FSpot.Tiff {
 		protected void LoadNextDirectory (System.IO.Stream stream)
 		{
 #if DEBUG_LOADER
-			System.Console.WriteLine ("start_position = {1} next_directory_offset = {0}", next_directory_offset, orig_position);
+			System.Console.WriteLine ("start_position = {1} next_directory_offset = {0}",
+						  next_directory_offset, orig_position);
 #endif
 			next_directory = null;
 			try {
@@ -1145,7 +1245,7 @@ namespace FSpot.Tiff {
 			}
 		}
 
-		public System.Collections.ArrayList Entries {
+		public List<DirectoryEntry> Entries {
 			get { 
 				return entries;
 			}
@@ -1156,10 +1256,18 @@ namespace FSpot.Tiff {
 			foreach (DirectoryEntry entry in entries)
 				if (entry.Id == id)
 					return entry;
+			
 
 			return null;
 		}
 
+		private DirectoryEntry GetEntry (int i)
+		{
+			if (i < Entries.Count)
+				return Entries [i];
+			else
+				return null;
+		}
 
 		public DirectoryEntry Lookup (uint id) 
 		{
@@ -1169,6 +1277,46 @@ namespace FSpot.Tiff {
 
 			return null;
 		}
+
+		public static DirectoryEntry CreateEntry (ImageDirectory parent, byte [] input, int start, Endian header_endian)
+		{
+			TagId tagid;
+			EntryType type;
+
+			DirectoryEntry.ParseHeader (input, start, out tagid, out type, header_endian);
+			//ConstructorFunc ctor = ctors[tagid];			
+			//if (ctor == null) {
+			//	return ctor (input, header_endian);				
+			//}
+			
+			switch (tagid) {
+			case TagId.ExifIfdPointer:
+			case TagId.GPSInfoIfdPointer:
+			case TagId.InteroperabilityIfdPointer:
+			case TagId.SubIFDs:
+				return new SubdirectoryEntry (input, start, header_endian);
+				//case TagId.MakerNote:
+				//return new MakerNoteEntry (input, start, header_endian);
+				//case TagId.PimIfdPointer:
+				//return new 
+				//case TagId.MakerNote:
+				//return new MakerNoteEntry (input, start, header_endian);
+			}
+			
+			switch (type) {
+			case EntryType.Ifd:
+				//System.Console.WriteLine ("Trying to load {0} {1}", tagid, tagid.ToString ("x"));
+				return new SubdirectoryEntry (input, start, header_endian);
+			case EntryType.Byte:
+				return new ByteEntry (input, start, header_endian);
+			case EntryType.Long:
+				return new LongEntry (input, start, header_endian);
+			case EntryType.Short:
+				return new ShortEntry (input, start, header_endian);
+			}
+
+			return new DirectoryEntry (input, start, header_endian);
+		}
 		
 		public Cms.Profile GetProfile ()
 		{
@@ -1198,7 +1346,6 @@ namespace FSpot.Tiff {
 						break;
 					}
 					break;
-
 				case TagId.WhitePoint:
 					Rational [] white = e.RationalValue;
 					whitepoint.x = white [0].Value;
@@ -1271,10 +1418,10 @@ namespace FSpot.Tiff {
 			System.Text.StringBuilder builder = new System.Text.StringBuilder ();
 			builder.Append ("Dummping IFD");
 			foreach (DirectoryEntry entry in entries) {
-				builder.Append (entry.ToString ()+ "\n");
+				builder.Append (entry.ToString ()+ Environment.NewLine);
 
 				if (entry is SubdirectoryEntry)
-					builder.Append ("Found SUBDIRECTORYENTRY\n");
+					builder.Append ("Found SUBDIRECTORYENTRY" + Environment.NewLine);
 			}
 			
 			if (next_directory != null) {
@@ -1286,51 +1433,6 @@ namespace FSpot.Tiff {
 		}
 	}
 	
-	public class EntryFactory {
-		//public delegate DirectoryEntry ConstructorFunc (byte [], Endian endian);
-		//public static System.Collections.Hashtable ctors = new System.Collections.Hashtable ();
-		
-		public static DirectoryEntry CreateEntry (ImageDirectory parent, byte [] input, int start, Endian header_endian)
-		{
-			TagId tagid;
-			EntryType type;
-
-			DirectoryEntry.ParseHeader (input, start, out tagid, out type, header_endian);
-			//ConstructorFunc ctor = ctors[tagid];			
-			//if (ctor == null) {
-			//	return ctor (input, header_endian);				
-			//}
-			
-			switch (tagid) {
-			case TagId.ExifIfdPointer:
-			case TagId.GPSInfoIfdPointer:
-			case TagId.InteroperabilityIfdPointer:
-			case TagId.SubIFDs:
-				return new SubdirectoryEntry (input, start, header_endian);
-				//case TagId.MakerNote:
-				//return new MakerNoteEntry (input, start, header_endian);
-				//case TagId.PimIfdPointer:
-				//return new 
-				//case TagId.MakerNote:
-				//return new MakerNoteEntry (input, start, header_endian);
-			}
-			
-			switch (type) {
-			case EntryType.Ifd:
-				//System.Console.WriteLine ("Trying to load {0} {1}", tagid, tagid.ToString ("x"));
-				return new SubdirectoryEntry (input, start, header_endian);
-			case EntryType.Byte:
-				return new ByteEntry (input, start, header_endian);
-			case EntryType.Long:
-				return new LongEntry (input, start, header_endian);
-			case EntryType.Short:
-				return new ShortEntry (input, start, header_endian);
-			}
-
-			return new DirectoryEntry (input, start, header_endian);
-		}
-	}
-	       
 	public class MakerNoteEntry : SubdirectoryEntry {
 		public MakerNoteEntry (byte [] data, int offset, Endian endian) : base (data, offset, endian)
 		{
@@ -1359,6 +1461,34 @@ namespace FSpot.Tiff {
 			}
 		}
 
+		public override uint Save (OrderedWriter writer, uint position)
+		{
+#if DEBUG_LOADER			
+			Console.WriteLine ("writing entry {0} {1} {2} - value offset = {3}", Id, Type, Count, position);
+#endif
+
+			writer.Write ((ushort)Id);
+			writer.Write ((ushort)Type);
+			writer.Write ((uint)Count);
+
+
+			if (Directory.Length == 1) {
+				writer.Write ((uint)position);
+				position = Directory [0].Save (writer, position);
+			} else if (Directory.Length > 1) {
+				writer.Write ((uint)position);
+				uint value_position = (uint) (position + Directory.Length * 4);
+				for (int i = 0; i < Directory.Length; i++) {
+					writer.Stream.Position = position + i * 4;
+					value_position = Directory [i].Save (writer, value_position);
+				}
+				return value_position;
+			} else
+				writer.Write ((uint) 0);
+
+			return position;
+		}
+
 		public virtual uint GetEntryCount ()
 		{
 			return count;
@@ -1376,8 +1506,8 @@ namespace FSpot.Tiff {
 					directory_offset = BitConverter.ToUInt32 (raw_data, i * 4, endian == Endian.Little);
 					Directory [i] = new ImageDirectory (stream, directory_offset, endian);
 				} catch (System.Exception e) {
-					System.Console.WriteLine ("Error loading Subdirectory {0} at {2} of {3}bytes:\n{1}", 
-								  this.Id, e, directory_offset, stream.Length);
+					System.Console.WriteLine ("Error loading Subdirectory {0} at {2} of {3}bytes:{4}{1}", 
+								  this.Id, e, directory_offset, stream.Length, Environment.NewLine);
 				}
 					
 			}
@@ -1494,6 +1624,27 @@ namespace FSpot.Tiff {
 			}
 		}
 
+		public virtual uint Save (OrderedWriter writer, uint position)
+		{
+#if DEBUG_LOADER			
+			Console.WriteLine ("writing entry {0} {1} {2}", Id, Type, Count);
+#endif
+			writer.Write ((ushort)Id);
+			writer.Write ((ushort)Type);
+			writer.Write ((uint)Count);
+			if (Length > 4) {
+				writer.Write ((uint)position);
+				writer.Stream.Position = position;
+				writer.Stream.Write (RawData, 0, RawData.Length);
+				return (uint) (position + RawData.Length);
+			} else {
+				writer.Stream.Write (RawData, 0, RawData.Length);
+				for (int i = 0; i < 4 - RawData.Length; i++)
+					writer.Write ((byte)0);
+			}
+			return position;
+		}
+
 		public void SetOrigin (uint pos)
 		{
 			offset_origin = pos;
@@ -1505,6 +1656,10 @@ namespace FSpot.Tiff {
 			}
 		}
 
+		public uint Length {
+			get { return (uint)(Count * GetTypeSize ()); }
+		}
+
 		public virtual int GetTypeSize ()
 		{
 			return GetTypeSize (type);
@@ -1560,12 +1715,16 @@ namespace FSpot.Tiff {
 			if (data_offset != 0) {
 				stream.Seek ((long)Position, System.IO.SeekOrigin.Begin);
 				byte [] data = new byte [count * GetTypeSize ()];
-				if (stream.Read (data, 0, data.Length) < data.Length)
+				if (stream.Read (data, 0, data.Length) < data.Length) {
+#if DEBUG_LOADER
+					System.Console.WriteLine ("Short read");
+#endif
 					throw new ShortReadException ();
+				}
 				raw_data = data;
 			}
 
-#if false
+#if DEBUG_LOADER
 			switch ((int)this.Id) {
 			case (int)TagId.NewSubfileType:
 				System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new NewSubFileType {0}", (NewSubfileType) this.ValueAsLong [0]);
@@ -1607,7 +1766,7 @@ namespace FSpot.Tiff {
 						System.Console.Write (" {0}", val);
 					break;
 				}
-				System.Console.WriteLine ("");
+				System.Console.WriteLine (String.Empty);
 				break;
 			}
 #endif
@@ -2032,6 +2191,9 @@ namespace FSpot.Tiff {
 			 */
 			sub = (SubdirectoryEntry) Header.Directory.Lookup (TagId.SubIFDs);	
 
+			if (e == null)
+				e = dir.Lookup (TagId.NewSubfileType);
+
 			int i = 0;
 			do {
 				uint dirtype = e.ValueAsLong [0];
@@ -2090,7 +2252,8 @@ namespace FSpot.Tiff {
 					break;
 
 				dir = sub.Directory [i];
-				e = dir.Lookup (TagId.NewSubfileType);
+				if (dir != null)
+					e = dir.Lookup (TagId.NewSubfileType);
 				i++;
 			} while (i < sub.Directory.Length);
 		}
@@ -2147,5 +2310,79 @@ namespace FSpot.Tiff {
 			return file;
 		}
 	}
+
+#if ENABLE_NUNIT
+	[TestFixture]
+	public class Tests {
+		public Tests ()
+		{
+			Gnome.Vfs.Vfs.Initialize ();
+			Gtk.Application.Init ();
+		}
+		
+		[Test]
+		public void Save ()
+		{
+			string desc = "this is an example description";
+			string desc2 = "\x00a9 Novell Inc.";
+			PixbufOrientation orient = PixbufOrientation.TopRight;
+			Gdk.Pixbuf test = new Gdk.Pixbuf (null, "f-spot-32.png");
+			string path = ImageFile.TempPath ("joe.jpg");
+			
+			PixbufUtils.SaveJpeg (test, path, 75, new Exif.ExifData ());
+			JpegFile jimg = new JpegFile (path);
+			jimg.SetDescription (desc);
+			jimg.SetOrientation (orient);
+			jimg.SaveMetaData (path);
+			JpegFile mod = new JpegFile (path);
+			Assert.AreEqual (mod.Orientation, orient);
+			Assert.AreEqual (mod.Description, desc);
+			jimg.SetDescription (desc2);
+			jimg.SaveMetaData (path);
+			mod = new JpegFile (path);
+			Assert.AreEqual (mod.Description, desc2);
+			
+			Header header = mod.ExifHeader;
+#if USE_TEST_FILE
+			string tmp = "/home/lewing/test.tiff";
+			if (File.Exists (tmp))
+				File.Delete (tmp);
+			Stream stream = File.Open (tmp, FileMode.Create, FileAccess.ReadWrite);
+			Console.WriteLine ("XXXX saving tiff {0}", tmp);
+#else
+			System.IO.MemoryStream stream = new System.IO.MemoryStream ();
+#endif
+
+			header.Dump ("source");
+			header.Save (stream);
+			stream.Position = 0;
+			System.Console.WriteLine ("----------------------------------------------LOADING TIFF");
+			Header loader = new Header (stream);
+			loader.Dump ("loader");
+			
+			CompareDirectories (header.Directory, loader.Directory);
+
+			System.IO.File.Delete (path);	
+		}
+
+		private void CompareDirectories (ImageDirectory olddir, ImageDirectory newdir)
+		{
+			Assert.AreEqual (olddir.Entries.Count, newdir.Entries.Count);
+			for (int i = 0; i < olddir.Entries.Count; i++) {
+				Assert.AreEqual (olddir.Entries [i].Id, newdir.Entries [i].Id);
+				Assert.AreEqual (olddir.Entries [i].Type, newdir.Entries [i].Type);
+				Assert.AreEqual (olddir.Entries [i].Count, newdir.Entries [i].Count);
+				Assert.AreEqual (olddir.Entries [i].Length, newdir.Entries [i].Length);
+				if (olddir.Entries [i] is SubdirectoryEntry) {
+					SubdirectoryEntry oldsub = olddir.Entries [i] as SubdirectoryEntry;
+					SubdirectoryEntry newsub = newdir.Entries [i] as SubdirectoryEntry;
+					
+					for (int j = 0; j < oldsub.Directory.Length; j++)
+						CompareDirectories (oldsub.Directory [j], newsub.Directory [j]);
+				}
+			}
+		}
+	}
+#endif
 }
 
