@@ -169,6 +169,9 @@ public class MainWindow {
 	};
 
 	private static TargetEntry [] icon_dest_target_table = new TargetEntry [] {
+#if ENABLE_REPARENTING
+		new TargetEntry ("application/x-fspot-photos", 0, (uint) TargetType.PhotoList),
+#endif
 		new TargetEntry ("application/x-fspot-tags", 0, (uint) TargetType.TagList),
 		new TargetEntry ("text/uri-list", 0, (uint) TargetType.UriList),
 	};
@@ -374,6 +377,12 @@ public class MainWindow {
 		icon_view.DragBegin += HandleIconViewDragBegin;
 		icon_view.DragDataGet += HandleIconViewDragDataGet;
 
+		TagMenu tag_menu = new TagMenu (null, Database.Tags);
+		tag_menu.NewTagHandler += delegate { HandleCreateTagAndAttach (this, null); };
+		tag_menu.TagSelected += HandleAttachTagMenuSelected;
+		tag_menu.Populate();
+		attach_tag.Submenu = tag_menu;
+		
 		PhotoTagMenu pmenu = new PhotoTagMenu ();
 		pmenu.TagSelected += HandleRemoveTagMenuSelected;
 		remove_tag.Submenu = pmenu;
@@ -455,6 +464,8 @@ public class MainWindow {
 		LoadPreference (Preferences.SHOW_TOOLBAR);
 		LoadPreference (Preferences.SHOW_SIDEBAR);
 		LoadPreference (Preferences.SHOW_TIMELINE);
+
+		LoadPreference (Preferences.GNOME_MAILTO_ENABLED);
 		
 		Preferences.SettingChanged += OnPreferencesChanged;
 
@@ -584,7 +595,7 @@ public class MainWindow {
 		}
 	}
 
-	public int [] SelectedIds () {
+	private int [] SelectedIds () {
 		int [] ids = new int [0];
 
 		if (fsview != null && fsview.View.Item.IsValid)
@@ -765,7 +776,7 @@ public class MainWindow {
 		return photo_list;
 	}
 
-	private Photo [] SelectedPhotos () 
+	public Photo [] SelectedPhotos () 
 	{
 		return SelectedPhotos (SelectedIds ());
 	}
@@ -830,16 +841,14 @@ public class MainWindow {
 	void HandleTagSelectionRowActivated (object sender, RowActivatedArgs args)
 	{
 		ShowQueryWidget ();
-		//query_widget.Require (new Tag [] {tag_selection_widget.TagByPath (args.Path)});
 		query_widget.Include (new Tag [] {tag_selection_widget.TagByPath (args.Path)});
 	}
 
 	void HandleTagSelectionButtonPressEvent (object sender, ButtonPressEventArgs args)
 	{
-		if (args.Event.Button == 3)
-		{
+		if (args.Event.Button == 3) {
 			TagPopup popup = new TagPopup ();
-			popup.Activate (args.Event, tag_selection_widget.TagAtPosition ((int)args.Event.X, (int)args.Event.Y),
+			popup.Activate (args.Event, tag_selection_widget.TagAtPosition (args.Event.X, args.Event.Y),
 			tag_selection_widget.TagHighlight);
 			args.RetVal = true;
 		}
@@ -919,78 +928,102 @@ public class MainWindow {
 	public void HandleTagSelectionDragMotion (object o, DragMotionArgs args)
 	{
 		TreePath path;
+        TreeViewDropPosition position = TreeViewDropPosition.IntoOrAfter;
+		tag_selection_widget.GetPathAtPos (args.X, args.Y, out path);
 
-		if (!tag_selection_widget.GetPathAtPos (args.X, args.Y, out path))
-			return;
+        if (path == null)
+            return;
 
-		tag_selection_widget.SetDragDestRow (path, Gtk.TreeViewDropPosition.IntoOrAfter);
+        // Tags can be dropped before, after, or into another tag
+        if (args.Context.Targets[0].Name == "application/x-fspot-tags") {
+            Gdk.Rectangle rect = tag_selection_widget.GetCellArea(path, tag_selection_widget.Columns[0]);
+            double vpos = Math.Abs(rect.Y - args.Y) / (double)rect.Height;
+            if (vpos < 0.2) {
+                position = TreeViewDropPosition.Before;
+            } else if (vpos > 0.8) {
+                position = TreeViewDropPosition.After;
+            }
+        }
 
-		//Scroll if required
+		tag_selection_widget.SetDragDestRow (path, position);
+
+		// Scroll if within 20 pixels of the top or bottom of the tag list
 		if (args.Y < 20)
 			tag_selection_scrolled.Vadjustment.Value -= 30;
-		if (((o as Gtk.Widget).Allocation.Height - args.Y) < 20)
+        else if (((o as Gtk.Widget).Allocation.Height - args.Y) < 20)
 			tag_selection_scrolled.Vadjustment.Value += 30;
 	}
 
 	public void HandleTagSelectionDragDataReceived (object o, DragDataReceivedArgs args)
 	{
-		Tag [] tags = new Tag [1];
+        TreePath path;
+        TreeViewDropPosition position;
+        if (!tag_selection_widget.GetDestRowAtPos((int)args.X, (int)args.Y, out path, out position))
+            return;
 
-		//FIXME this is a lame api, we need to fix the drop behaviour of these things
-		tags [0] = tag_selection_widget.TagAtPosition(args.X, args.Y);
-
-		if (tags [0] == null)
+        Tag tag = path == null ? null : tag_selection_widget.TagByPath (path);
+		if (tag == null)
 			return;
 
 		switch (args.Info) {
 		case (uint)TargetType.PhotoList:
 			db.BeginTransaction ();
 			foreach (int num in SelectedIds ()) {
-				AddTagExtended (num, tags);
+				AddTagExtended (num, new Tag[] {tag});
 			}
 			db.CommitTransaction ();
-			query_widget.PhotoTagsChanged (tags);
+			query_widget.PhotoTagsChanged (new Tag[] {tag});
 			break;
 		case (uint)TargetType.UriList:
 			UriList list = new UriList (args.SelectionData);
 			
 			db.BeginTransaction ();
-			foreach (string path in list.ToLocalPaths ()) {
-				Photo photo = db.Photos.GetByPath (path);
+			foreach (string photo_path in list.ToLocalPaths ()) {
+				Photo photo = db.Photos.GetByPath (photo_path);
 				
 				// FIXME - at this point we should import the photo, and then continue
 				if (photo == null)
 					continue;
 				
 				// FIXME this should really follow the AddTagsExtended path too
-				photo.AddTag (tags);
+				photo.AddTag (new Tag[] {tag});
+				db.Photos.Commit (photo);
 			}
 			db.CommitTransaction ();
 			InvalidateViews ();
 			break;
 		case (uint)TargetType.TagList:
-			if (tag_selection_widget.TagHighlight.Length < 1)
+			Category parent;
+            if (position == TreeViewDropPosition.Before || position == TreeViewDropPosition.After) {
+                parent = tag.Category;
+            } else {
+                parent = tag as Category;
+            }
+
+			if (parent == null || tag_selection_widget.TagHighlight.Length < 1) {
+                args.RetVal = false;
 				return;
+            }
 
-			Tag child = tag_selection_widget.TagHighlight [0];
-			Tag parent = tags[0];
+            int moved_count = 0;
+            Tag [] highlighted_tags = tag_selection_widget.TagHighlight;
+			foreach (Tag child in tag_selection_widget.TagHighlight) {
+                // FIXME with this reparenting via dnd, you cannot move a tag to root.
+                if (child != parent && child.Category != parent && !child.IsAncestorOf(parent)) {
+                    child.Category = parent as Category;
 
-			// FIXME with this reparenting via dnd, you cannot move a tag to root.
-			if (child != parent && !child.IsAncestorOf(parent) && child.Category != parent && parent is Category)
-			{
-				child.Category = parent as Category;
+                    // Saving changes will automatically cause the TreeView to be updated
+                    db.Tags.Commit (child);
+                    moved_count++;
+                }
+            }
 
-				// Saving changes will automatically cause the TreeView to be updated
-				db.Tags.Commit (child);
-				
-				args.RetVal = true;
-			} else {
-				args.RetVal = false;
-			}
+            // Reselect the same tags
+            tag_selection_widget.TagHighlight = highlighted_tags;
 
+            args.RetVal = moved_count > 0;
 			break;
 		}
-
 	}
 
 #if SHOW_CALENDAR
@@ -1240,6 +1273,21 @@ public class MainWindow {
 			UriList list = new UriList (args.SelectionData);
 			ImportUriList (list, (args.Context.Action & Gdk.DragAction.Copy) != 0);
 			break;
+#if ENABLE_REPARENTING
+		case (uint)TargetType.PhotoList:
+			int p_item = icon_view.CellAtPosition (args.X + (int) icon_view.Hadjustment.Value, 
+							     args.Y + (int) icon_view.Vadjustment.Value);
+
+			if (p_item >= 0)
+			{
+				PhotoVersionCommands.Reparent cmd = new PhotoVersionCommands.Reparent ();
+				
+				cmd.Execute (db.Photos, SelectedPhotos(), query.Photos [p_item], GetToplevel (null));
+				UpdateQuery ();
+			}
+	
+			break;
+#endif
 		}
 
 		Gtk.Drag.Finish (args.Context, true, false, args.Time);
@@ -1446,8 +1494,8 @@ public class MainWindow {
 				cam.InitializeCamera ();
 
 				FSpot.CameraFileSelectionDialog selector = new FSpot.CameraFileSelectionDialog (cam, db);
-				selector.Run ();
-
+				if (selector.Run() > 0)
+					query.RollSet = new RollSet (db.Rolls.GetRolls (1)[0]);
 				UpdateQuery ();
 			}
 		}
@@ -1986,7 +2034,7 @@ public class MainWindow {
 					Gdk.Pixbuf orig = FSpot.PhotoLoader.Load (query, id);
 					Gdk.Pixbuf final = PixbufUtils.UnsharpMask (orig, radius_spin.Value, amount_spin.Value, threshold_spin.Value);
 					
-					bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
+					bool create_version = photo.DefaultVersion.IsProtected;
 
 					photo.SaveVersion (final, create_version);
 					query.Commit (id);
@@ -2361,10 +2409,12 @@ public class MainWindow {
  			HandleDeleteSelectedTagCommand (sender, (EventArgs) args);
 			break;
 		
-		/*case Gdk.Key.space:
+		case Gdk.Key.space:
 		case Gdk.Key.Return:
-			tag_selection_widget.ToggleSelectedTags ();
-			break;*/
+		case Gdk.Key.KP_Enter:
+            ShowQueryWidget ();
+            query_widget.Include (tag_selection_widget.TagHighlight);
+			break;
 
 		case Gdk.Key.F2:
 			tag_selection_widget.EditSelectedTagName ();
@@ -2374,8 +2424,6 @@ public class MainWindow {
 			args.RetVal = false;
 			break;
 		}
-
-		return;
 	}
 
 	public void HandleDeleteSelectedTagCommand (object sender, EventArgs args)
@@ -2625,6 +2673,9 @@ public class MainWindow {
 		case Preferences.METADATA_EMBED_IN_IMAGE:
 			write_metadata = (bool) val;
 			break;
+		case Preferences.GNOME_MAILTO_ENABLED:
+			send_mail.Visible = (bool) val;
+			break;
 		}
 	}
 
@@ -2651,7 +2702,7 @@ public class MainWindow {
 
 	// Queries.
 
-	void UpdateQuery ()
+	public void UpdateQuery ()
 	{
 		main_window.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Watch);
 		main_window.GdkWindow.Display.Sync ();
