@@ -1,20 +1,23 @@
 using Gnome;
 using System;
 using System.Collections;
+using FSpot.Query;
 
 namespace FSpot {
 	public class PhotoQuery : FSpot.IBrowsableCollection {
 		private Photo [] photos;
 		private PhotoStore store;
+		private Term terms;
 		private Tag [] tags;
 		private string extra_condition;
 		private PhotoStore.DateRange range = null;
+		private RollSet roll_set = null;
 		
 		// Constructor
 		public PhotoQuery (PhotoStore store)
 		{
 			this.store = store;
-			photos = store.Query (null, null, range);
+			photos = store.Query ((Tag [])null, null, range, roll_set);
 		}
 
 		public int Count {
@@ -29,6 +32,7 @@ namespace FSpot {
 
 		// IPhotoCollection Interface
 		public event FSpot.IBrowsableCollectionChangedHandler Changed;
+		public event FSpot.IBrowsableCollectionChangedHandler PreChanged;
 		public event FSpot.IBrowsableCollectionItemsChangedHandler ItemsChanged;
 		
 		public IBrowsableItem this [int index] {
@@ -55,13 +59,12 @@ namespace FSpot {
 			}
 		}
 		
-		public Tag [] Tags {
+		public Term Terms {
 			get {
-				return tags;
+				return terms;
 			}
-			
 			set {
-				tags = value;
+				terms = value;
 				untagged = false;
 				RequestReload ();
 			}
@@ -115,12 +118,27 @@ namespace FSpot {
 			}
 		}
 
+		public RollSet RollSet {
+			get { return roll_set; }
+			set {
+				if (value == roll_set)
+					return;
+
+				roll_set = value;	
+ 				RequestReload ();
+ 			}
+		}
+
 		public void RequestReload ()
 		{
 			if (untagged)
-				photos = store.QueryUntagged (range);
+				photos = store.QueryUntagged (range, roll_set);
 			else
-				photos = store.Query (tags, extra_condition, range);
+				photos = store.Query (terms, extra_condition, range, roll_set);
+
+			//this event will allow resorting the query content
+			if (PreChanged != null)
+				PreChanged (this);
 
 			if (Changed != null)
 				Changed (this);
