@@ -1,4 +1,16 @@
+/*
+ * PhotoStore.cs
+ *
+ * Author(s):
+	Ettore Perazzoli <ettore@perazzoli.org>
+	Larry Ewing <lewing@gnome.org>
+	Stephane Delcroix <stephane@delcroix.org>
+ * 
+ * This is free software. See COPYING for details.
+ */
+
 using Gnome;
+using Gnome.Vfs;
 using Gdk;
 using Gtk;
 
@@ -6,6 +18,7 @@ using Mono.Data.SqliteClient;
 using Mono.Unix;
 
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 using System.Text;
 using System;
@@ -14,10 +27,14 @@ using FSpot.Query;
 
 using Banshee.Database;
 
-public class PhotoVersion : FSpot.IBrowsableItem {
+public class PhotoVersion : FSpot.IBrowsableItem
+{
 	Photo photo;
 	uint version_id;
-	
+	System.Uri uri;
+	string name;
+	bool is_protected;
+
 	public System.DateTime Time {
 		get { return photo.Time; }
 	}
@@ -26,8 +43,8 @@ public class PhotoVersion : FSpot.IBrowsableItem {
 		get { return photo.Tags; }
 	}
 
-	public Uri DefaultVersionUri {
-		get { return UriList.PathToFileUri (photo.GetVersionPath (version_id));}
+	public System.Uri DefaultVersionUri {
+		get { return uri; }
 	}
 
 	public string Description {
@@ -35,21 +52,38 @@ public class PhotoVersion : FSpot.IBrowsableItem {
 	}
 
 	public string Name {
-		get { return photo.GetVersionName (version_id); }
+		get { return name; }
+		set { name = value; }
 	}
 
 	public Photo Photo {
 		get { return photo; }
 	}
 
+	public System.Uri Uri {
+		get { return uri; }
+		set { 
+			if (value == null)
+				throw new System.ArgumentNullException ("uri");
+			uri = value;
+		}
+	}
+
 	public uint VersionId {
 		get { return version_id; }
 	}
 
-	public PhotoVersion (Photo photo, uint version_id)
+	public bool IsProtected {
+		get { return is_protected; }
+	}
+
+	public PhotoVersion (Photo photo, uint version_id, System.Uri uri, string name, bool is_protected)
 	{
 		this.photo = photo;
 		this.version_id = version_id;
+		this.uri = uri;
+		this.name = name;
+		this.is_protected = is_protected;
 	}
 }
 
@@ -101,16 +135,18 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 
 	private static int CompareCurrentDir (Photo photo1, Photo photo2)
 	{
-		return string.Compare (photo1.directory_path, photo2.directory_path);
+		return string.Compare (photo1.DirectoryPath, photo2.DirectoryPath);
 	}
 
 	private static int CompareName (Photo photo1, Photo photo2)
 	{
-		return string.Compare (photo1.name, photo2.name);
+		return string.Compare (photo1.Name, photo2.Name);
 	}
 
-	public class CompareDateName : IComparer {
-		public int Compare (object obj1, object obj2) {
+	public class CompareDateName : IComparer
+	{
+		public int Compare (object obj1, object obj2)
+		{
 			Photo p1 = (Photo)obj1;
 			Photo p2 = (Photo)obj2;
 
@@ -121,11 +157,12 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 
 			return result;
 		}
-
 	}
 
-	public class CompareDirectory : IComparer {
-		public int Compare (object obj1, object obj2) {
+	public class CompareDirectory : IComparer
+	{
+		public int Compare (object obj1, object obj2)
+		{
 			Photo p1 = (Photo)obj1;
 			Photo p2 = (Photo)obj2;
 
@@ -138,10 +175,12 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		}
 	}
 
-	public class RandomSort : IComparer {
+	public class RandomSort : IComparer
+	{
 		Random random = new Random ();
 		
-		public int Compare (object obj1, object obj2) {
+		public int Compare (object obj1, object obj2)
+		{
 			return random.Next (-5, 5);
 		}
 	}
@@ -149,35 +188,22 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	// The time is always in UTC.
 	private DateTime time;
 	public DateTime Time {
-		get {
-			return time;
-		}
-		set {
-			time = value;
-		}
-	}
-
-	private string directory_path;
-	private string name;
-
-	private string Path {
-		get {
-			return System.IO.Path.Combine (directory_path, name);
-		}
+		get { return time; }
+		set { time = value; }
 	}
 
 	public string Name {
-		get {
-			return name;
-		}
+		get { return System.IO.Path.GetFileName (VersionUri (OriginalVersionId).AbsolutePath); }
 	}
 
+	//This property no longer keeps a 'directory' path, but the logical container for the image, like:
+	// file:///home/bob/Photos/2007/08/23 or
+	// http://www.google.com/logos
+	[Obsolete ("MARKED FOR REMOVAL. no longer makes sense with versions in different Directories. Any way to get rid of this ?")]
 	public string DirectoryPath {
-		get {
-			return directory_path;
-		}
-		set {
-			directory_path = value;
+		get { 
+			System.Uri uri = VersionUri (OriginalVersionId);
+			return uri.Scheme + "://" + uri.Host + System.IO.Path.GetDirectoryName (uri.AbsolutePath);
 		}
 	}
 
@@ -186,29 +212,21 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		get {
 			if (tags == null)
 				return new Tag [0];
-			else 
-				return (Tag []) tags.ToArray (typeof (Tag));
+
+			return (Tag []) tags.ToArray (typeof (Tag));
 		}
 	}
 
 	private bool loaded = false;
 	public bool Loaded {
-		get {
-			return loaded;
-		}
-		set {
-			loaded = value;
-		}
+		get { return loaded; }
+		set { loaded = value; }
 	}
 
 	private string description;
 	public string Description {
-		get {
-			return description;
-		}
-		set {
-			description = value;
-		}
+		get { return description; }
+		set { description = value; }
 	}
 
 	private uint roll_id = 0;
@@ -221,18 +239,22 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	public const int OriginalVersionId = 1;
 	private uint highest_version_id;
 
-	private Hashtable version_names;
+	private Dictionary<uint, PhotoVersion> versions;
+	private Dictionary<uint, PhotoVersion> Versions {
+		get {
+			if (versions == null)
+				versions = new Dictionary<uint, PhotoVersion> ();
+			return versions;
+		}
+	}
 
 	public uint [] VersionIds {
 		get {
-			if (version_names == null)
+			if (versions == null)
 				return new uint [0];
 
-			uint [] ids = new uint [version_names.Count];
-			uint i = 0;
-			foreach (uint id in version_names.Keys)
-				ids [i ++] = id;
-
+			uint [] ids = new uint [versions.Count];
+			versions.Keys.CopyTo (ids, 0);
 			Array.Sort (ids);
 			return ids;
 		}
@@ -240,146 +262,204 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 
 	public IBrowsableItem GetVersion (uint version_id)
 	{
-		return new PhotoVersion (this, version_id);
+		if (versions == null)
+			return null;
+
+		return versions [version_id];
 	}
 
 	private uint default_version_id = OriginalVersionId;
 	public uint DefaultVersionId {
-		get {
-			return default_version_id;
-		}
-
-		set {
-			default_version_id = value;
-		}
+		get { return default_version_id; }
+		set { default_version_id = value; }
 	}
 
 	// This doesn't check if a version of that name already exists, 
 	// it's supposed to be used only within the Photo and PhotoStore classes.
-	public void AddVersionUnsafely (uint version_id, string name)
+	internal void AddVersionUnsafely (uint version_id, System.Uri uri, string name, bool is_protected)
 	{
-		if (version_names == null)
-			version_names = new Hashtable ();
-
-		version_names [version_id] = name;
+		Versions [version_id] = new PhotoVersion (this, version_id, uri, name, is_protected);
 
 		highest_version_id = Math.Max (version_id, highest_version_id);
 	}
 
-	private string GetPathForVersionName (string version_name)
+	public uint AddVersion (System.Uri uri, string name)
 	{
-		string name_without_extension = System.IO.Path.GetFileNameWithoutExtension (name);
-		string extension = System.IO.Path.GetExtension (name);
+		return AddVersion (uri, name, false);
+	}
 
-		return System.IO.Path.Combine (directory_path,  name_without_extension 
-					       + " (" + version_name + ")" + extension);
+	public uint AddVersion (System.Uri uri, string name, bool is_protected)
+	{
+		if (VersionNameExists (name))
+			throw new ApplicationException ("A version with that name already exists");
+		highest_version_id ++;
+		Versions [highest_version_id] = new PhotoVersion (this, highest_version_id, uri, name, is_protected);
+		return highest_version_id;
+	}
+
+	//FIXME: store versions next to originals. will crash on ro locations.
+	private System.Uri GetUriForVersionName (string version_name, string extension)
+	{
+		string name_without_extension = System.IO.Path.GetFileNameWithoutExtension (Name);
+
+		return new System.Uri (System.IO.Path.Combine (DirectoryPath,  name_without_extension 
+					       + " (" + version_name + ")" + extension));
 	}
 
 	public bool VersionNameExists (string version_name)
 	{
-		if (version_names != null) {
-			foreach (string n in version_names.Values) {
-				if (n == version_name)
-					return true;
-			}
-		}
+		foreach (PhotoVersion v in Versions.Values)
+			if (v.Name == version_name)
+				return true;
 
 		return false;
 	}
 
+	[Obsolete ("use GetVersion (uint).Name")]
 	public string GetVersionName (uint version_id)
 	{
-		if (version_names != null)
-			return version_names [version_id] as string;
-		else 
-			return null;
+		PhotoVersion v = GetVersion (version_id) as PhotoVersion;
+		if (v != null)
+			return v.Name;
+		return null;
 	}
 
+	[Obsolete ("Use VersionUri (uint) instead")]
         public string GetVersionPath (uint version_id)
 	{
-		if (version_id == OriginalVersionId)
-			return Path;
-		else
-			return GetPathForVersionName (GetVersionName (version_id));
+		return VersionUri (version_id).LocalPath;
 	}
 
 	public System.Uri VersionUri (uint version_id)
 	{
-		return UriList.PathToFileUri (GetVersionPath (version_id));
+		if (!Versions.ContainsKey (version_id))
+			return null;
+
+		PhotoVersion v = Versions [version_id]; 
+		if (v != null)
+			return v.Uri;
+
+		return null;
 	}
 	
 	public System.Uri DefaultVersionUri {
-		get {
-			return VersionUri (DefaultVersionId);
-		}
+		get { return VersionUri (DefaultVersionId); }
 	}
 
+	public PhotoVersion DefaultVersion {
+		get {
+			if (!Versions.ContainsKey (DefaultVersionId))
+				return null;
+			return Versions [DefaultVersionId]; 
+		}
+	}
 	public void DeleteVersion (uint version_id)
 	{
-		DeleteVersion (version_id, false);
+		DeleteVersion (version_id, false, false);
 	}
 
 	public void DeleteVersion (uint version_id, bool remove_original)
+	{
+		DeleteVersion (version_id, remove_original, false);
+	}
+
+	public void DeleteVersion (uint version_id, bool remove_original, bool keep_file)
 	{
 		if (version_id == OriginalVersionId && !remove_original)
 			throw new Exception ("Cannot delete original version");
 
-		string path = GetVersionPath (version_id);
-		if (File.Exists (path))
-			File.Delete (path);
+		System.Uri uri =  VersionUri (version_id);
 
-		try {
-			string thumb_path = ThumbnailGenerator.ThumbnailPath (path);
-			File.Delete (thumb_path);
-		} catch (System.Exception) {
-			//ignore an error here we don't really care.
+		if (!keep_file) {
+			if ((new Gnome.Vfs.Uri (uri.ToString ())).Exists)
+				(new Gnome.Vfs.Uri (uri.ToString ())).Unlink ();	
+
+			try {
+				string thumb_path = ThumbnailGenerator.ThumbnailPath (uri);
+				System.IO.File.Delete (thumb_path);
+			} catch (System.Exception) {
+				//ignore an error here we don't really care.
+			}
+			PhotoStore.DeleteThumbnail (uri);
 		}
-
-		PhotoStore.DeleteThumbnail (path);
-
-		version_names.Remove (version_id);
+		Versions.Remove (version_id);
 
 		do {
 			version_id --;
-			if (version_names.Contains (version_id)) {
+			if (Versions.ContainsKey (version_id)) {
 				DefaultVersionId = version_id;
 				break;
 			}
 		} while (version_id > OriginalVersionId);
 	}
 
-	public uint CreateVersion (string name, uint base_version_id, bool create_file)
+	public uint CreateProtectedVersion (string name, uint base_version_id, bool create)
 	{
-		string new_path = GetPathForVersionName (name);
-		string original_path = GetVersionPath (base_version_id);
+		return CreateVersion (name, base_version_id, create, true);
+	}
+
+	public uint CreateVersion (string name, uint base_version_id, bool create)
+	{
+		return CreateVersion (name, base_version_id, create, false);
+	}
+
+	private uint CreateVersion (string name, uint base_version_id, bool create, bool is_protected)
+	{
+		System.Uri new_uri = GetUriForVersionName (name, System.IO.Path.GetExtension (VersionUri (base_version_id).AbsolutePath));
+		System.Uri original_uri = VersionUri (base_version_id);
 
 		if (VersionNameExists (name))
 			throw new Exception ("This version name already exists");
 
-		if (File.Exists (new_path))
-			throw new Exception (String.Format ("A file named {0} already exists",
-							    System.IO.Path.GetFileName (new_path)));
+		if ((new Gnome.Vfs.Uri (new_uri.ToString ())).Exists)
+			throw new Exception (String.Format ("An object at this uri {0} already exists", new_uri.ToString ()));
 
-		if (create_file) {
-			Mono.Unix.Native.Stat stat;
-			int stat_err = Mono.Unix.Native.Syscall.stat (original_path, out stat);
-			File.Copy (original_path, new_path);
-			FSpot.ThumbnailGenerator.Create (new_path).Dispose ();
-			
-			if (stat_err == 0) 
-				try {
-					Mono.Unix.Native.Syscall.chown(new_path, Mono.Unix.Native.Syscall.getuid (), stat.st_gid);
-				} catch (Exception) {}
+		if (create) {
+			Xfer.XferUri (
+				new Gnome.Vfs.Uri (original_uri.ToString ()), 
+				new Gnome.Vfs.Uri (new_uri.ToString ()),
+				XferOptions.Default, XferErrorMode.Abort, 
+				XferOverwriteMode.Abort, 
+				delegate (Gnome.Vfs.XferProgressInfo info) {return 1;});
+
+//			Mono.Unix.Native.Stat stat;
+//			int stat_err = Mono.Unix.Native.Syscall.stat (original_path, out stat);
+//			File.Copy (original_path, new_path);
+			FSpot.ThumbnailGenerator.Create (new_uri).Dispose ();
+//			
+//			if (stat_err == 0) 
+//				try {
+//					Mono.Unix.Native.Syscall.chown(new_path, Mono.Unix.Native.Syscall.getuid (), stat.st_gid);
+//				} catch (Exception) {}
 		}
-		if (version_names == null)
-			version_names = new Hashtable ();
-
 		highest_version_id ++;
-		version_names [highest_version_id] = name;
+		Versions [highest_version_id] = new PhotoVersion (this, highest_version_id, new_uri, name, is_protected);
 
 		return highest_version_id;
 	}
 
+	public uint CreateReparentedVersion (PhotoVersion version)
+	{
+		return CreateReparentedVersion (version, false);
+	}
+
+	public uint CreateReparentedVersion (PhotoVersion version, bool is_protected)
+	{
+		int num = 0;
+		while (true) {
+			num++;
+			string name = Mono.Posix.Catalog.GetPluralString ("Reparented", "Reparented ({0})", num);
+			name = String.Format (name, num);
+			if (VersionNameExists (name))
+				continue;
+
+			highest_version_id ++;
+			Versions [highest_version_id] = new PhotoVersion (this, highest_version_id, version.Uri, name, is_protected);
+
+			return highest_version_id;
+		}
+	}
+
 	public uint CreateDefaultModifiedVersion (uint base_version_id, bool create_file)
 	{
 		int num = 1;
@@ -425,18 +505,15 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		if (VersionNameExists (new_name))
 			throw new Exception ("This name already exists");
 
-		string original_name = version_names [version_id] as string;
+		(GetVersion (version_id) as PhotoVersion).Name = new_name;
 
-		string old_path = GetPathForVersionName (original_name);
-		string new_path = GetPathForVersionName (new_name);
+		//TODO: rename file too ???
 
-		if (File.Exists (new_path))
-			throw new Exception ("File with this name already exists");
-
-		File.Move (old_path, new_path);
-		PhotoStore.MoveThumbnail (old_path, new_path);
-
-		version_names [version_id] = new_name;
+//		if (System.IO.File.Exists (new_path))
+//			throw new Exception ("File with this name already exists");
+//
+//		File.Move (old_path, new_path);
+//		PhotoStore.MoveThumbnail (old_path, new_path);
 	}
 
 
@@ -517,6 +594,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		return xmp;
 	}
 
+	//FIXME: Won't work on non-file uris
 	public void WriteMetadataToImage ()
 	{
 		string path = this.DefaultVersionUri.LocalPath;
@@ -543,6 +621,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		}
 	}
 
+	//FIXME: won't work on non file uris
 	public uint SaveVersion (Gdk.Pixbuf buffer, bool create_version)
 	{
 		uint version = DefaultVersionId;
@@ -559,7 +638,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 			try {
 				string version_path = GetVersionPath (version);
 			
-				using (Stream stream = File.OpenWrite (version_path)) {
+				using (Stream stream = System.IO.File.OpenWrite (version_path)) {
 					img.Save (buffer, stream);
 				}
 				FSpot.ThumbnailGenerator.Create (version_path).Dispose ();
@@ -577,26 +656,30 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	}
 
 	// Constructor
-	public Photo (uint id, long unix_time, string directory_path, string name)
+	public Photo (uint id, long unix_time, System.Uri uri)
 		: base (id)
 	{
-		time = DbUtils.DateTimeFromUnixTime (unix_time);
+		if (uri == null)
+			throw new System.ArgumentNullException ("uri");
 
-		this.directory_path = directory_path;
-		this.name = name;
+		time = DbUtils.DateTimeFromUnixTime (unix_time);
 
 		description = String.Empty;
 
 		// Note that the original version is never stored in the photo_versions table in the
 		// database.
-		AddVersionUnsafely (OriginalVersionId, Catalog.GetString ("Original"));
+		AddVersionUnsafely (OriginalVersionId, uri, Catalog.GetString ("Original"), true);
 	}
 
-	public Photo (uint id, long unix_time, string path)
-		: this (id, unix_time,
+	[Obsolete ("Use Photo (uint, long, Uri) instead")]
+	public Photo (uint id, long unix_time, string directory_path, string name)
+		: this (id, unix_time, System.IO.Path.Combine (directory_path, name))
+	{
+	}
 
-			System.IO.Path.GetDirectoryName (path),
-			System.IO.Path.GetFileName (path))
+	[Obsolete ("Use Photo (uint, long, Uri) instead")]
+	public Photo (uint id, long unix_time, string path)
+		: this (id, unix_time, UriList.PathToFileUri (path))
 	{
 	}
 
@@ -623,22 +706,22 @@ public class PhotoStore : DbStore {
 		string large_thumbnail_file_name_template = Thumbnail.PathForUri ("file:///boo", ThumbnailSize.Large);
 		string large_thumbnail_directory_path = System.IO.Path.GetDirectoryName (large_thumbnail_file_name_template);
 
-		if (! File.Exists (large_thumbnail_directory_path))
-			Directory.CreateDirectory (large_thumbnail_directory_path);
+		if (! System.IO.File.Exists (large_thumbnail_directory_path))
+			System.IO.Directory.CreateDirectory (large_thumbnail_directory_path);
 	}
 
 	//
 	// Generates the thumbnail, returns the Pixbuf, and also stores it as a side effect
 	//
 
-	public static Pixbuf GenerateThumbnail (Uri uri)
+	public static Pixbuf GenerateThumbnail (System.Uri uri)
 	{
 		using (FSpot.ImageFile img = FSpot.ImageFile.Create (uri)) {
 			return GenerateThumbnail (uri, img);
 		}
 	}
 
-	public static Pixbuf GenerateThumbnail (Uri uri, ImageFile img)
+	public static Pixbuf GenerateThumbnail (System.Uri uri, ImageFile img)
 	{
 		Pixbuf thumbnail = null;
 
@@ -660,17 +743,22 @@ public class PhotoStore : DbStore {
 		return thumbnail;
 	}
 
-	public static void DeleteThumbnail (string path)
+//	[Obsolete ("use DeleteThumbnail (System.Uri) instead")]
+//	public static void DeleteThumbnail (string path)
+//	{
+//		DeleteThumbnail (UriList.PathToFileUri (path));
+//	}
+
+	public static void DeleteThumbnail (System.Uri uri)
 	{
-		string uri = UriList.PathToFileUri (path).ToString ();
-		path = Thumbnail.PathForUri (uri, ThumbnailSize.Large);
-		if (File.Exists (path))
-			File.Delete (path);
+		string path = Thumbnail.PathForUri (uri.ToString (), ThumbnailSize.Large);
+		if (System.IO.File.Exists (path))
+			System.IO.File.Delete (path);
 	}
 
 	public static void MoveThumbnail (string old_path, string new_path)
 	{
-		File.Move (ThumbnailGenerator.ThumbnailPath (UriList.PathToFileUri (old_path)),
+		System.IO.File.Move (ThumbnailGenerator.ThumbnailPath (UriList.PathToFileUri (old_path)),
 			   ThumbnailGenerator.ThumbnailPath(UriList.PathToFileUri (new_path)));
 	}
 
@@ -686,14 +774,15 @@ public class PhotoStore : DbStore {
 			return;
 		
 		Database.ExecuteNonQuery ( 
+			//WARNING: if you change this schema, reflect your changes 
+			//to Updater.cs, at revision 7.0
 			"CREATE TABLE photos (                                     " +
 			"	id                 INTEGER PRIMARY KEY NOT NULL,   " +
-			"       time               INTEGER NOT NULL,	   	   " +
-			"       directory_path     STRING NOT NULL,		   " +
-			"       name               STRING NOT NULL,		   " +
-			"       description        TEXT NOT NULL,	           " +
-			"       roll_id            INTEGER NOT NULL,		   " +
-			"       default_version_id INTEGER NOT NULL		   " +
+			"	time               INTEGER NOT NULL,	   	   " +
+			"	uri		   STRING NOT NULL,		   " +
+			"	description        TEXT NOT NULL,	           " +
+			"	roll_id            INTEGER NOT NULL,		   " +
+			"	default_version_id INTEGER NOT NULL		   " +
 			")");
 
 
@@ -705,43 +794,56 @@ public class PhotoStore : DbStore {
 
 
 		Database.ExecuteNonQuery (
-			"CREATE TABLE photo_versions (    " +
-			"       photo_id        INTEGER,  " +
-			"       version_id      INTEGER,  " +
-			"       name            STRING    " +
+			//WARNING: if you change this schema, reflect your changes 
+			//to Updater.cs, at revision 8.0
+			"CREATE TABLE photo_versions (    	" +
+			"       photo_id        INTEGER,  	" +
+			"       version_id      INTEGER,  	" +
+			"       name            STRING,    	" +
+			"	uri		STRING NOT NULL," +
+			"	protected	BOOLEAN		" +
 			")");
 	}
 
 
+	[Obsolete ("Use Create (Uri, uint, out Pixbuf) instead")]
 	public Photo Create (string path, uint roll_id, out Pixbuf thumbnail)
 	{
 		return Create (path, path, roll_id, out thumbnail);
-
 	}
 
-	public Photo Create (string newPath, string origPath, uint roll_id, out Pixbuf thumbnail)
+	[Obsolete ("Use Create (Uri, Uri, uint, out Pixbuf) instead")]
+	public Photo Create (string new_path, string orig_path, uint roll_id, out Pixbuf thumbnail)
+	{
+		return Create (UriList.PathToFileUri (new_path), UriList.PathToFileUri (orig_path), roll_id, out thumbnail);
+	}
+
+	public Photo Create (System.Uri uri, uint roll_id, out Pixbuf thumbnail)
+	{
+		return Create (uri, uri, roll_id, out thumbnail);
+	}
+
+	public Photo Create (System.Uri new_uri, System.Uri orig_uri, uint roll_id, out Pixbuf thumbnail)
 	{
 		Photo photo;
-		using (FSpot.ImageFile img = FSpot.ImageFile.Create (origPath)) {
+		using (FSpot.ImageFile img = FSpot.ImageFile.Create (orig_uri)) {
 			long unix_time = DbUtils.UnixTimeFromDateTime (img.Date);
 			string description = img.Description != null  ? img.Description.Split ('\0') [0] : String.Empty;
 	
-	 		uint id = (uint) Database.Execute (new DbCommand ("INSERT INTO photos (time, "	+
-					"directory_path, name, description, roll_id, default_version_id) "	+
-	 				"VALUES (:time, :directory_path, :name, :description, "		+
-					":roll_id, :default_version_id)",
-	 				"time", unix_time,
-	 				"directory_path", System.IO.Path.GetDirectoryName (newPath),
-	 				"name", System.IO.Path.GetFileName (newPath),
-	 				"description", description,
-					"roll_id", roll_id,
-	 				"default_version_id", Photo.OriginalVersionId));
+	 		uint id = (uint) Database.Execute (new DbCommand (
+				"INSERT INTO photos (time, uri, description, roll_id, default_version_id) "	+
+	 			"VALUES (:time, :uri, :description, :roll_id, :default_version_id)",
+	 			"time", unix_time,
+				"uri", new_uri.ToString (),
+	 			"description", description,
+				"roll_id", roll_id,
+	 			"default_version_id", Photo.OriginalVersionId));
 	
-			photo = new Photo (id, unix_time, newPath);
+			photo = new Photo (id, unix_time, new_uri);
 			AddToCache (photo);
 			photo.Loaded = true;
 	
-			thumbnail = GenerateThumbnail (UriList.PathToFileUri (newPath), img);		
+			thumbnail = GenerateThumbnail (new_uri, img);		
 			EmitAdded (photo);
 		}
 		return photo;
@@ -749,19 +851,21 @@ public class PhotoStore : DbStore {
 
 	private void GetVersions (Photo photo)
 	{
-		SqliteDataReader reader = Database.Query(new DbCommand("SELECT version_id, name FROM photo_versions WHERE photo_id = :id", photo.Id));
+		SqliteDataReader reader = Database.Query(new DbCommand("SELECT version_id, name, uri, protected FROM photo_versions WHERE photo_id = :id", "id", photo.Id));
 
 		while (reader.Read ()) {
 			uint version_id = Convert.ToUInt32 (reader [0]);
 			string name = reader[1].ToString ();
-			photo.AddVersionUnsafely (version_id, name);
+			System.Uri uri = new System.Uri (reader[2].ToString ());
+			bool is_protected = Convert.ToBoolean (reader[3]);
+			photo.AddVersionUnsafely (version_id, uri, name, is_protected);
 		}
 		reader.Close();
 	}
 
 	private void GetTags (Photo photo)
 	{
-		SqliteDataReader reader = Database.Query(new DbCommand("SELECT tag_id FROM photo_tags WHERE photo_id = :id", photo.Id));
+		SqliteDataReader reader = Database.Query(new DbCommand("SELECT tag_id FROM photo_tags WHERE photo_id = :id", "id", photo.Id));
 
 		while (reader.Read ()) {
 			uint tag_id = Convert.ToUInt32 (reader [0]);
@@ -772,7 +876,7 @@ public class PhotoStore : DbStore {
 	}		
 	
 	private void GetAllVersions  () {
-		SqliteDataReader reader = Database.Query("SELECT photo_id, version_id, name FROM photo_versions");
+		SqliteDataReader reader = Database.Query("SELECT photo_id, version_id, name, uri, protected FROM photo_versions");
 		
 		while (reader.Read ()) {
 			uint id = Convert.ToUInt32 (reader [0]);
@@ -791,8 +895,9 @@ public class PhotoStore : DbStore {
 			if (reader [1] != null) {
 				uint version_id = Convert.ToUInt32 (reader [1]);
 				string name = reader[2].ToString ();
-				
-				photo.AddVersionUnsafely (version_id, name);
+				System.Uri uri = new System.Uri (reader[3].ToString ());
+				bool is_protected = Convert.ToBoolean (reader[4]);	
+				photo.AddVersionUnsafely (version_id, uri, name, is_protected);
 			}
 
 			/*
@@ -831,43 +936,9 @@ public class PhotoStore : DbStore {
 		reader.Close();
 	}
 
-	private void GetAllData () {
-		SqliteDataReader reader = Database.Query("SELECT photo_tags.photo_id, tag_id, version_id, name "
-                                               + "FROM photo_tags, photo_versions "
-                                               + "WHERE photo_tags.photo_id = photo_versions.photo_id");
-
-		while (reader.Read ()) {
-			uint id = Convert.ToUInt32 (reader [0]);
-			Photo photo = LookupInCache (id) as Photo;
-				
-			if (photo == null) {
-				//Console.WriteLine ("Photo {0} not found", id);
-				continue;
-			}
-				
-			if (photo.Loaded) {
-				//Console.WriteLine ("Photo {0} already Loaded", photo);
-				continue;
-			}
-
-		        if (reader [1] != null) {
-				uint tag_id = Convert.ToUInt32 (reader [1]);
-				Tag tag = Core.Database.Tags.Get (tag_id) as Tag;
-				photo.AddTagUnsafely (tag);
-			}
-			if (reader [2] != null) {
-				uint version_id = Convert.ToUInt32 (reader [2]);
-				string name = reader[3].ToString ();
-				
-				photo.AddVersionUnsafely (version_id, name);
-			}
-		}
-		reader.Close();
-	}
-
 	private void GetData (Photo photo)
 	{
-		SqliteDataReader reader = Database.Query(new DbCommand("SELECT tag_id, version_id, name "
+		SqliteDataReader reader = Database.Query(new DbCommand("SELECT tag_id, version_id, name, uri, protected "
                                                              + "FROM photo_tags, photo_versions "
                                                              + "WHERE photo_tags.photo_id = photo_versions.photo_id "
                                                              + "AND photo_tags.photo_id = :id", "id", photo.Id));
@@ -881,8 +952,9 @@ public class PhotoStore : DbStore {
 			if (reader [1] != null) {
 				uint version_id = Convert.ToUInt32 (reader [1]);
 				string name = reader[2].ToString ();
-				
-				photo.AddVersionUnsafely (version_id, name);
+				System.Uri uri = new System.Uri (reader[3].ToString ());
+				bool is_protected = Convert.ToBoolean (reader[4]);	
+				photo.AddVersionUnsafely (version_id, uri, name, is_protected);
 			}
 		}
 		reader.Close();
@@ -894,18 +966,17 @@ public class PhotoStore : DbStore {
 		if (photo != null)
 			return photo;
 
-		SqliteDataReader reader = Database.Query(new DbCommand("SELECT time, directory_path, name, description, "
-                                                     + "roll_id, default_version_id FROM photos WHERE id = :id", "id", id));
+		SqliteDataReader reader = Database.Query(new DbCommand("SELECT time, uri, description, roll_id, default_version_id "
+			+ "FROM photos WHERE id = :id", "id", id));
 
 		if (reader.Read ()) {
 			photo = new Photo (id,
-					   Convert.ToInt64 (reader [0]),
-					   reader [1].ToString (),
-					   reader [2].ToString ());
+				Convert.ToInt64 (reader [0]),
+				new System.Uri (reader [1].ToString ()));
 
-			photo.Description = reader[3].ToString ();
-			photo.RollId = Convert.ToUInt32 (reader[4]);
-			photo.DefaultVersionId = Convert.ToUInt32 (reader[5]);
+			photo.Description = reader[2].ToString ();
+			photo.RollId = Convert.ToUInt32 (reader[3]);
+			photo.DefaultVersionId = Convert.ToUInt32 (reader[4]);
 			AddToCache (photo);
 		}
 		reader.Close();
@@ -919,35 +990,38 @@ public class PhotoStore : DbStore {
 		return photo;
 	}
 
+	[Obsolete ("Use GetByUri instead")]
 	public Photo GetByPath (string path)
 	{
-		//FIXME - No cacheing here - probably not a problem since
-		//        this is only used for DND
+		return GetByUri (UriList.PathToFileUri (path));
+	}
 
+	public Photo GetByUri (System.Uri uri)
+	{
 		Photo photo = null;
 
-		string directory_path = System.IO.Path.GetDirectoryName (path);
-		string filename = System.IO.Path.GetFileName (path);
-
-		SqliteDataReader reader = Database.Query(new DbCommand("SELECT id, time, description, roll_id, default_version_id FROM photos "
-                + "WHERE directory_path = :directory_path AND name = :name", "directory_path", directory_path, "name", filename));
+		SqliteDataReader reader = Database.Query (new DbCommand ("SELECT id, time, description, roll_id, default_version_id FROM photos "
+                + "WHERE uri = :uri", "uri", uri.ToString ()));
 
 		if (reader.Read ()) {
 			photo = new Photo (Convert.ToUInt32 (reader [0]),
 					   Convert.ToInt64 (reader [1]),
-					   directory_path,
-					   filename);
+					   uri);
 
 			photo.Description = reader[2].ToString ();
 			photo.RollId = Convert.ToUInt32 (reader[3]);
 			photo.DefaultVersionId = Convert.ToUInt32 (reader[4]);
-			AddToCache (photo);
 		}
-        reader.Close();
+	        reader.Close();
 
 		if (photo == null)
 			return null;
-		
+
+		if (LookupInCache (photo.Id) as Photo != null)
+			return LookupInCache (photo.Id) as Photo;
+
+		AddToCache (photo);
+	
 		GetTags (photo);
 		GetVersions (photo);
 
@@ -956,7 +1030,7 @@ public class PhotoStore : DbStore {
 
 	public void Remove (Tag []tags)
 	{
-		Photo [] photos = Query (tags);	
+		Photo [] photos = Query (tags, String.Empty, null, null);	
 
 		foreach (Photo photo in photos) {
 			photo.RemoveCategory (tags);
@@ -1007,9 +1081,9 @@ public class PhotoStore : DbStore {
 		if (items.Length > 1)
 			Database.BeginTransaction ();
 
-		foreach (DbItem item in items) {
+		foreach (DbItem item in items)
 			Update ((Photo)item);
-		}
+		
 		EmitChanged (items, args);
 
 		if (items.Length > 1)
@@ -1019,14 +1093,17 @@ public class PhotoStore : DbStore {
 	private void Update (Photo photo) {
 		// Update photo.
 
-		Database.ExecuteNonQuery (new DbCommand ("UPDATE photos SET description = :description, " + 
-						     "directory_path = :directory_path, " +
-						     "default_version_id = :default_version_id, time = :time WHERE id = :id ",
-						     "description", photo.Description,
-						     "directory_path", photo.DirectoryPath,
-						     "default_version_id", photo.DefaultVersionId,
-						     "time", DbUtils.UnixTimeFromDateTime (photo.Time),
-						     "id", photo.Id));
+		Database.ExecuteNonQuery (new DbCommand (
+			"UPDATE photos SET description = :description, " + 
+			"default_version_id = :default_version_id, " + 
+			"time = :time, " + 
+			"uri = :uri " +
+			"WHERE id = :id ",
+			"description", photo.Description,
+			"default_version_id", photo.DefaultVersionId,
+			"time", DbUtils.UnixTimeFromDateTime (photo.Time),
+			"uri", photo.VersionUri (Photo.OriginalVersionId).ToString (),
+			"id", photo.Id));
 
 		// Update tags.
 
@@ -1045,12 +1122,12 @@ public class PhotoStore : DbStore {
 		foreach (uint version_id in photo.VersionIds) {
 			if (version_id == Photo.OriginalVersionId)
 				continue;
+			PhotoVersion version = photo.GetVersion (version_id) as PhotoVersion;
 
-			string version_name = photo.GetVersionName (version_id);
-
-			Database.ExecuteNonQuery(new DbCommand ("INSERT INTO photo_versions (photo_id, version_id, name) " +
-							     "       VALUES (:photo_id, :version_id, :name)",
-							     "photo_id", photo.Id, "version_id", version_id, "name", version_name));
+			Database.ExecuteNonQuery(new DbCommand ("INSERT INTO photo_versions (photo_id, version_id, name, uri, protected) " +
+							     "       VALUES (:photo_id, :version_id, :name, :uri, :is_protected)",
+							     "photo_id", photo.Id, "version_id", version.VersionId,
+							     "name", version.Name, "uri", version.Uri.ToString (), "is_protected", version.IsProtected));
 		}
 	}
 	
@@ -1123,12 +1200,11 @@ public class PhotoStore : DbStore {
 			if (photo == null) {
 				photo = new Photo (id,
 						   Convert.ToInt64 (reader [1]),
-						   reader [2].ToString (),
-						   reader [3].ToString ());
+						   new System.Uri (reader [2].ToString ()));
 				
-				photo.Description = reader[4].ToString ();
-				photo.RollId = Convert.ToUInt32 (reader[5]);
-				photo.DefaultVersionId = Convert.ToUInt32 (reader[6]);
+				photo.Description = reader[3].ToString ();
+				photo.RollId = Convert.ToUInt32 (reader[4]);
+				photo.DefaultVersionId = Convert.ToUInt32 (reader[5]);
 				
 				version_list.Add (photo);
 			}
@@ -1156,17 +1232,19 @@ public class PhotoStore : DbStore {
 		return id_list.ToArray (typeof (Photo)) as Photo [];
 	}
 
+	[Obsolete ("No longer make any sense with uris...")]
 	public Photo [] Query (System.IO.DirectoryInfo dir)
 	{
-		string query_string = String.Format ("SELECT photos.id,                          " +
-						     "       photos.time,                        " +
-						     "       photos.directory_path,              " +
-						     "       photos.name,                        " +
-						     "       photos.description,                 " +
-						     "       photos.roll_id,                     " +
-						     "       photos.default_version_id           " +
-						     "     FROM photos                           " +
-						     "     WHERE directory_path = \"{0}\"", dir.FullName);
+		string query_string = String.Format (
+			"SELECT photos.id, "			+
+				"photos.time, "			+
+				"photos.uri, "			+
+				"photos.description, "		+
+				"photos.roll_id, "		+
+				"photos.default_version_id "	+
+			"FROM photos " 				+
+			"WHERE uri LIKE \"file://{0}%\" "	+
+			"AND uri NOT LIKE \"file://{0}/%/%\"" , dir.FullName);
 
 		return Query (query_string);
 	}
@@ -1209,8 +1287,7 @@ public class PhotoStore : DbStore {
 		//
 		// SELECT photos.id
 		//        photos.time
-		//        photos.directory_path,
-		//        photos.name,
+		//        photos.uri,
 		//        photos.description,
 		//	  photos.roll_id,
 		//        photos.default_version_id
@@ -1227,8 +1304,7 @@ public class PhotoStore : DbStore {
 		StringBuilder query_builder = new StringBuilder ();
 		query_builder.Append ("SELECT photos.id, " 			+
 					     "photos.time, "			+
-					     "photos.directory_path, " 		+
-					     "photos.name, "			+
+					     "photos.uri, "			+
 					     "photos.description, "		+
 				      	     "photos.roll_id, "   		+
 					     "photos.default_version_id "	+
@@ -1276,7 +1352,7 @@ public class PhotoStore : DbStore {
 #if TEST_PHOTO_STORE
 	static void Dump (Photo photo)
 	{
-		Console.WriteLine ("\t[{0}] {1}", photo.Id, photo.Path);
+	//	Console.WriteLine ("\t[{0}] {1}", photo.Id, photo.Path);
 		Console.WriteLine ("\t{0}", photo.Time.ToLocalTime ());
 
 		if (photo.Description != String.Empty)
