@@ -206,4 +206,34 @@ public class PhotoVersionCommands {
 			return true;
 		}
 	}
+
+	// Reparenting a photo as version of another one
+	public class Reparent {
+		public bool Execute (PhotoStore store, Photo [] photos, Photo new_parent, Gtk.Window parent_window)
+		{
+			foreach (Photo photo in photos) {
+				new_parent.AddTag (photo.Tags);
+				foreach (uint version_id in photo.VersionIds) {
+					try {
+						new_parent.DefaultVersionId = new_parent.CreateReparentedVersion (photo.GetVersion (version_id) as PhotoVersion);
+						store.Commit (new_parent);
+					} catch (Exception e) {
+						Console.WriteLine (e);	
+					}
+				}
+				uint [] version_ids = photo.VersionIds;
+				Array.Reverse (version_ids);
+				foreach (uint version_id in version_ids) {
+					try {
+						photo.DeleteVersion (version_id, true, true);
+					} catch (Exception e) {
+						Console.WriteLine(e);
+					}
+				}
+				store.Commit (photo);
+				MainWindow.Toplevel.Database.Photos.Remove (photos);
+			}
+			return true;
+		}
+	}
 }
