@@ -18,8 +18,7 @@ namespace FSpot {
 		[Glade.Widget] Gtk.MenuItem zoom_in;
 		[Glade.Widget] Gtk.MenuItem zoom_out;
 
-		[Glade.Widget] Gtk.Image near_image;
-		[Glade.Widget] Gtk.Image far_image;
+		[Glade.Widget] Gtk.MenuItem export;
 
 		[Glade.Widget] Gtk.Scale zoom_scale;
 
@@ -35,8 +34,6 @@ namespace FSpot {
 		
 		UriCollection collection;
 		
-		FSpot.Delay slide_delay;
-
 		FullScreenView fsview;
 
 		private static Gtk.Tooltips toolTips = new Gtk.Tooltips ();
@@ -72,17 +69,28 @@ namespace FSpot {
 
 			Gtk.Toolbar toolbar = new Gtk.Toolbar ();
 			toolbar_hbox.PackStart (toolbar);
-			Widget rl_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-270", new System.EventHandler (HandleRotate270Command));
-			SetTip (rl_button, Catalog.GetString ("Rotate photo left"));
-			Widget rr_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-90", new System.EventHandler (HandleRotate90Command));
-			SetTip (rr_button, Catalog.GetString ("Rotate photo right"));
+		
+			ToolButton rl_button = GtkUtil.ToolButtonFromTheme ("object-rotate-left", Catalog.GetString ("Rotate Left"), true);
+			rl_button.Clicked += HandleRotate270Command;
+			rl_button.SetTooltip (toolTips, Catalog.GetString ("Rotate photo left"), null);
+			toolbar.Insert (rl_button, -1);
 
-			toolbar.AppendSpace ();
+			ToolButton rr_button = GtkUtil.ToolButtonFromTheme ("object-rotate-right", Catalog.GetString ("Rotate Right"), true);
+			rr_button.Clicked += HandleRotate90Command;
+			rr_button.SetTooltip (toolTips, Catalog.GetString ("Rotate photo right"), null);
+			toolbar.Insert (rr_button, -1);
 
-			Widget fs_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-fullscreen", new System.EventHandler (HandleViewFullscreen));
-			SetTip (fs_button, Catalog.GetString ("View photos fullscreen"));
-			Widget ss_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-slideshow", new System.EventHandler (HandleViewSlideshow));
-			SetTip (ss_button, Catalog.GetString ("View photos in a slideshow"));
+			toolbar.Insert (new SeparatorToolItem (), -1);
+
+			ToolButton fs_button = GtkUtil.ToolButtonFromTheme ("view-fullscreen", Catalog.GetString ("Fullscreen"), true);
+			fs_button.Clicked += HandleViewFullscreen;
+			fs_button.SetTooltip (toolTips, Catalog.GetString ("View photos fullscreen"), null);
+			toolbar.Insert (fs_button, -1);
+
+			ToolButton ss_button = GtkUtil.ToolButtonFromTheme ("media-playback-start", Catalog.GetString ("Slideshow"), true);
+			ss_button.Clicked += HandleViewSlideshow;
+			ss_button.SetTooltip (toolTips, Catalog.GetString ("View photos in a slideshow"), null);
+			toolbar.Insert (ss_button, -1);
 
 			collection = new UriCollection (uris);
 
@@ -122,11 +130,6 @@ namespace FSpot {
 
 			ShowSidebar = collection.Count > 1;
 
-			near_image.SetFromStock ("f-spot-stock_near", Gtk.IconSize.SmallToolbar);
-			far_image.SetFromStock ("f-spot-stock_far", Gtk.IconSize.SmallToolbar);
-
-			slide_delay = new FSpot.Delay (new GLib.IdleHandler (SlideShow));
-
 			LoadPreference (Preferences.VIEWER_SHOW_FILENAMES);
 
 			Preferences.SettingChanged += OnPreferencesChanged;
@@ -138,11 +141,19 @@ namespace FSpot {
 			if (collection.Count > 0)
 				directory_view.Selection.Add (0);
 
+			export.Submenu = (Mono.Addins.AddinManager.GetExtensionNode ("/FSpot/Menus/Exports") as FSpot.Extensions.SubmenuNode).GetMenuItem ().Submenu;
+			export.Submenu.ShowAll ();
+			export.Activated += HandleExportActivated ;
+		}
+
+		void HandleExportActivated (object o, EventArgs e)
+		{
+			FSpot.Extensions.ExportMenuItemNode.SelectedImages = delegate () {return new FSpot.PhotoArray (directory_view.Selection.Items); };
 		}
 
 		public void HandleCollectionChanged (IBrowsableCollection collection)
 		{
-			if (collection.Count > 0) {
+			if (collection.Count > 0 && directory_view.Selection.Count == 0) {
 				Console.WriteLine ("Added selection");
 				directory_view.Selection.Add (0);
 			}
@@ -203,46 +214,12 @@ namespace FSpot {
 			}
 		}		
 
-		void HandleExportToFlickr (object sender, System.EventArgs args)
-		{
-			new FSpot.FlickrExport (FlickrNet.SupportedService.Flickr, new PhotoArray (directory_view.Selection.Items), false);
-		}
-
-		void HandleExportTo23hq (object sender, System.EventArgs args)
-		{
-			new FSpot.FlickrExport (FlickrNet.SupportedService.TwentyThreeHQ, new PhotoArray (directory_view.Selection.Items), false);
-		}
-
-		void HandleExportToPicasaweb (object sender, System.EventArgs args)
-		{
-			new FSpot.GoogleExport (new PhotoArray (directory_view.Selection.Items));
-		}
-
-		void HandleExportToSmugMug (object sender, System.EventArgs args)
-		{
-			new FSpot.SmugMugExport (new PhotoArray (directory_view.Selection.Items)); 	
-		}
-
-		void HandleExportToGallery (object sender, System.EventArgs args)
-		{
-			new FSpot.GalleryExport (new PhotoArray (directory_view.Selection.Items)); 
-		}
-
-		void HandleExportToFolder (object sender, System.EventArgs args)
-		{
-			new FSpot.FolderExport (new PhotoArray (directory_view.Selection.Items));			
-		}
-
-		void HandleExportToCD (object sender, System.EventArgs args)
-		{
-			new FSpot.CDExport (new PhotoArray (directory_view.Selection.Items)); 
-		}
-
 		private void HandleSelectionChanged (FSpot.IBrowsableCollection selection) 
 		{
+			
 			if (selection.Count > 0) {
 				image_view.Item.Index = ((IconView.SelectionCollection)selection).Ids[0];
-				image_view.Reload ();
+
 				zoom_scale.Value = image_view.NormalizedZoom;
 				if (metadata_dialog != null)
 					metadata_dialog.InfoDisplay.Photo = image_view.Item.Current;
@@ -287,8 +264,15 @@ namespace FSpot {
 
 		private void HandleViewSlideshow (object sender, System.EventArgs args)
 		{
-			this.Window.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Watch);
-			slide_delay.Start ();
+			if (fsview != null)
+				fsview.Destroy ();
+
+			fsview = new FSpot.FullScreenView (collection);
+			fsview.Destroyed += HandleFullScreenViewDestroy;
+
+			fsview.View.Item.Index = image_view.Item.Index;
+			fsview.Show ();
+			fsview.PlayPause ();
 		}
 	
 		private void HandleViewMetadata (object sender, System.EventArgs args)
@@ -441,8 +425,8 @@ namespace FSpot {
 			Gtk.Menu popup_menu = new Gtk.Menu ();
 			bool has_item = image_view.Item.Current != null;
 
-			GtkUtil.MakeMenuItem (popup_menu, "f-spot-rotate-270", delegate { HandleRotate270Command(window, null); }, has_item);
-			GtkUtil.MakeMenuItem (popup_menu, "f-spot-rotate-90", delegate { HandleRotate90Command (window, null); }, has_item);
+			GtkUtil.MakeMenuItem (popup_menu, Catalog.GetString ("Rotate _Left"), "object-rotate-left", delegate { HandleRotate270Command(window, null); }, has_item);
+			GtkUtil.MakeMenuItem (popup_menu, Catalog.GetString ("Rotate _Right"), "object-rotate-right", delegate { HandleRotate90Command (window, null); }, has_item);
 			GtkUtil.MakeMenuSeparator (popup_menu);
 			GtkUtil.MakeMenuItem (popup_menu, Catalog.GetString ("Set as Background"), HandleSetAsBackgroundCommand, has_item);
 
@@ -486,7 +470,6 @@ namespace FSpot {
 
 			sb.AppendFormat (Catalog.GetPluralString ("{0} Photo", "{0} Photos", collection.Count), collection.Count);
 			status_label.Text = sb.ToString ();
-
 		}
 
 		private void HandleFileClose (object sender, System.EventArgs args)
@@ -600,11 +583,6 @@ namespace FSpot {
 			}
 		}
 
-		public static void SetTip (Widget widget, string tip)
-		{
-			toolTips.SetTip (widget, tip, null);
-		}
-
 		public class PreferenceDialog : GladeDialog {
 			[Glade.Widget] private CheckButton interpolation_check;
 			[Glade.Widget] private ColorButton color_button;
