@@ -31,9 +31,9 @@ public class TagCommands {
 		[Glade.Widget] private Entry tag_name_entry;
 		[Glade.Widget] private Label prompt_label;
 		[Glade.Widget] private Label already_in_use_label;
-
-		[Glade.Widget]
-		private OptionMenu category_option_menu;
+		[Glade.Widget] private Label photo_label;
+		[Glade.Widget] private ScrolledWindow photo_scrolled_window;
+		[Glade.Widget] private OptionMenu category_option_menu;
 
 		Gtk.Widget parent_window;
 
@@ -334,12 +334,19 @@ public class TagCommands {
 		Gtk.IconView icon_view;
 		ListStore icon_store;
 		string icon_name = null;
+		Gtk.FileChooserButton external_photo_chooser;
+
 
 		[Glade.Widget] Gtk.Image preview_image;
 		[Glade.Widget] ScrolledWindow photo_scrolled_window;
 		[Glade.Widget] ScrolledWindow icon_scrolled_window;
 		[Glade.Widget] Label photo_label;
+		[Glade.Widget] Label from_photo_label;
+		[Glade.Widget] Label from_external_photo_label;
+		[Glade.Widget] private Label predefined_icon_label;
 		[Glade.Widget] SpinButton photo_spin_button;
+		[Glade.Widget] HBox external_photo_chooser_hbox;
+		[Glade.Widget] Button noicon_button;
 
 		private Gdk.Pixbuf PreviewPixbuf {
 			get { return preview_image.Pixbuf; }
@@ -372,6 +379,20 @@ public class TagCommands {
 			image_view.Item.Index = value;
 		}
 
+		private void HandleExternalFileSelectionChanged (object sender, EventArgs args)
+		{	//Note: The filter on the FileChooserButton's dialog means that we will have a Pixbuf compatible uri here
+			CreateTagIconFromExternalPhoto ();
+		}
+
+		private void CreateTagIconFromExternalPhoto ()
+		{
+			using (FSpot.ImageFile img = FSpot.ImageFile.Create(new Uri(external_photo_chooser.Uri))) {
+				using (Gdk.Pixbuf external_image = img.Load ()) {
+					PreviewPixbuf = PixbufUtils.TagIconFromPixbuf (external_image);
+				}
+			}
+		}
+
 		private void HandleSelectionChanged ()
 		{
 			int x, y, width, height;
@@ -405,6 +426,9 @@ public class TagCommands {
 
 		public void HandleIconSelectionChanged (object o, EventArgs args)
 		{
+			if (icon_view.SelectedItems.Length == 0)
+				return;
+
 			TreeIter iter;
 			icon_store.GetIter (out iter, icon_view.SelectedItems [0]); 
 			IconName = (string) icon_store.GetValue (iter, 0);
@@ -440,6 +464,17 @@ public class TagCommands {
 			image_view.SelectionChanged += HandleSelectionChanged;
 			image_view.PhotoChanged += HandlePhotoChanged;
 
+                        external_photo_chooser = new Gtk.FileChooserButton (Catalog.GetString ("Select Photo from file"),
+                                                                 Gtk.FileChooserAction.Open);
+
+			external_photo_chooser.Filter = new FileFilter();
+			external_photo_chooser.Filter.AddPixbufFormats();
+                        external_photo_chooser.LocalOnly = false;
+                        external_photo_chooser_hbox.PackStart (external_photo_chooser);
+
+    			Dialog.ShowAll ();
+			external_photo_chooser.SelectionChanged += HandleExternalFileSelectionChanged;
+
 			photo_scrolled_window.Add (image_view);
 
 			if (query.Photos.Length > 0) {
@@ -451,8 +486,15 @@ public class TagCommands {
 				
 				image_view.Item.Index = 0;
 			} else {
-				photo_spin_button.Sensitive = false;
-				photo_spin_button.Value = 0.0;
+				from_photo_label.Markup = String.Format (Catalog.GetString (
+					"\n<b>From Photo</b>\n" +
+					" You can use one of your library photos as an icon for this tag.\n" +
+					" However, first you must have at least one photo associated\n" +
+					" with this tag. Please tag a photo as '{0}' and return here\n" +
+					" to use it as an icon."), t.Name); 
+				photo_scrolled_window.Visible = false;
+				photo_label.Visible = false;
+				photo_spin_button.Visible = false;
 			}			
 
 			icon_store = new ListStore (typeof (string), typeof (Gdk.Pixbuf));
@@ -476,16 +518,21 @@ public class TagCommands {
 
 			if (response == ResponseType.Ok) {
 				try {
-					if (IconName != null)
+					if (IconName != null) {
 						t.ThemeIconName = IconName;
-					else
+					} else {
+						t.ThemeIconName = null;
 						t.Icon = PreviewPixbuf;
+					}
 					//db.Tags.Commit (t);
 					success = true;
 				} catch (Exception ex) {
 					// FIXME error dialog.
 					Console.WriteLine ("error {0}", ex);
 				}
+			} else if (response == (ResponseType) (1)) {
+				t.Icon = null;
+				success = true;
 			}
 			
 			this.Dialog.Destroy ();
