@@ -34,7 +34,7 @@ using System;
 
 using Mono.Unix;
 
-public class TagSelectionWidget : TreeView {
+public class TagSelectionWidget : FSpot.Widgets.SaneTreeView {
 	TagSelectionWidget widget;
 	private TagStore tag_store;
 
@@ -47,6 +47,11 @@ public class TagSelectionWidget : TreeView {
 
 	// Selection management.
 
+	public Tag TagAtPosition (double x, double y) 
+    {
+        return TagAtPosition((int) x, (int) y);
+    }
+
 	public Tag TagAtPosition (int x, int y) 
 	{
 		TreePath path;
@@ -186,7 +191,7 @@ public class TagSelectionWidget : TreeView {
 
 		// FIXME I can't set the Pixbuf to null, not sure if it's a GTK# bug...
 		if (tag.Icon != null)
-			(renderer as CellRendererPixbuf).Pixbuf = tag.Icon;
+			(renderer as CellRendererPixbuf).Pixbuf = tag.SizedIcon;
 		else
 			(renderer as CellRendererPixbuf).Pixbuf = empty_pixbuf;
 	}
@@ -449,9 +454,15 @@ public class TagSelectionWidget : TreeView {
 			}
 		}
 
-		if (expanded_tags.Count > 0)
-			FSpot.Preferences.Set (	FSpot.Preferences.EXPANDED_TAGS,
+#if GCONF_SHARP_2_18
+		FSpot.Preferences.Set (	FSpot.Preferences.EXPANDED_TAGS, (int []) expanded_tags.ToArray (typeof (int)));
+#else
+		if (expanded_tags.Count == 0)
+			expanded_tags.Add (-1);
+
+		FSpot.Preferences.Set (	FSpot.Preferences.EXPANDED_TAGS,
 						(int []) expanded_tags.ToArray (typeof (int)));
+#endif
 	}
 
 	public void EditSelectedTagName ()
@@ -467,16 +478,6 @@ public class TagSelectionWidget : TreeView {
 		text_render.Editable = false;
 	}
 
-	// Override the default ButtonPressEvent because it is stupid and
-	// returns true, meaning if you want to have another handler for the ButtonPress
-	// event you have to Glib.ConnectBefore.  Plus you usually want the default one to
-	// run first so it will select/unselect anything necessary before popping up a menu, say.
-	protected override bool OnButtonPressEvent (Gdk.EventButton button)
-	{
-		base.OnButtonPressEvent (button);
-		return false;
-	}
-	
 	public void HandleTagNameEdited (object sender, EditedArgs args)
 	{
 		args.RetVal = false;
@@ -531,7 +532,6 @@ public class TagSelectionWidget : TreeView {
 		: base (new TreeStore (typeof(uint), typeof(string)))
 	{
 		HeadersVisible = false;
-		Selection.Mode = SelectionMode.Multiple;
 
 		complete_column = new TreeViewColumn ();
 				
