@@ -23,12 +23,13 @@ namespace FSpot {
 		public static Gdk.Pixbuf Create (Uri uri)
 		{
 			try {
-				ImageFile img = ImageFile.Create (uri);
-				Gdk.Pixbuf thumb = img.Load (256, 256);
+				using (ImageFile img = ImageFile.Create (uri)) {
+					Gdk.Pixbuf thumb = img.Load (256, 256);
 
-				if (thumb != null)
-					Save (thumb, uri);
-				return thumb;
+					if (thumb != null)
+						Save (thumb, uri);
+					return thumb;
+				}
 			} catch {
 				return null;
 			}
@@ -42,6 +43,9 @@ namespace FSpot {
 				Gnome.Vfs.FileInfo vfs = new Gnome.Vfs.FileInfo (uri.ToString ());
 				DateTime mtime = vfs.Mtime;
 				valid  = Gnome.Thumbnail.IsValid (thumbnail, UriList.UriToStringEscaped (uri), mtime);
+			} catch (System.IO.FileNotFoundException) {
+				// If the original file is not on disk, the thumbnail is as valid as it's going to get
+				valid = true;
 			} catch (System.Exception e) {
 				System.Console.WriteLine (e);
 				valid = false;
