@@ -66,8 +66,8 @@ class Timer : IDisposable {
 }
 
 public class UriList : ArrayList {
-	public UriList (Photo [] photos) {
-		foreach (Photo p in photos) {
+	public UriList (FSpot.IBrowsableItem [] photos) {
+		foreach (FSpot.IBrowsableItem p in photos) {
 			Uri uri;
 			try {
 				uri = p.DefaultVersionUri;
@@ -256,7 +256,7 @@ public class UriList : ArrayList {
 			if (uri == null)
 				break;
 
-			list.Append (uri.ToString () + "\r\n");
+			list.Append (uri.ToString () + Environment.NewLine);
 		}
 
 		return list.ToString ();
@@ -312,12 +312,12 @@ class GtkUtil {
 		Gtk.ImageMenuItem i = new Gtk.ImageMenuItem (label);
 		i.Activated += e;
                 i.Sensitive = enabled;
-		i.Image = new Gtk.Image (image_name, Gtk.IconSize.Menu);
+		i.Image = Gtk.Image.NewFromIconName (image_name, Gtk.IconSize.Menu);
 		
 		menu.Append (i);
 		i.Show ();
 
-        return i;
+	        return i;
 	}
 
 	public static Gtk.MenuItem MakeCheckMenuItem (Gtk.Menu menu, string label, EventHandler e, bool enabled, bool active, bool as_radio)
@@ -356,55 +356,34 @@ class GtkUtil {
 		public object Sender;
 	}
 
-	public static Gtk.Widget MakeToolbarButton (Gtk.Toolbar toolbar, string stock_id, System.EventHandler e)
+	public static Gtk.ToolButton ToolButtonFromTheme (string theme_id, string label, bool important)
 	{
-		Gtk.StockItem item = Gtk.StockItem.Zero;
-		if (Gtk.StockManager.Lookup (stock_id, ref item)) {
-			SignalFuncHelper helper = new SignalFuncHelper (e);
-			Gtk.Widget w =  toolbar.AppendItem (item.Label.Replace ("_", null),
-							    null, null, 
-							    new Gtk.Image (item.StockId, Gtk.IconSize.LargeToolbar), 
-							    new Gtk.SignalFunc (helper.Func));
-			helper.Sender = w;
-			return w;
-		}
-		return null;
+		Gtk.ToolButton button = new Gtk.ToolButton (null, null);
+		button.Label = label;
+		button.IconName = theme_id;
+		button.IsImportant = important;
+		button.UseUnderline = true;
+		return button;
 	}
-	
-	public static Gtk.Widget MakeToolbarToggleButton (Gtk.Toolbar toolbar, string stock_id, System.EventHandler e)
+
+	public static Gdk.Pixbuf TryLoadIcon (Gtk.IconTheme theme, string icon_name, int size, Gtk.IconLookupFlags flags)
 	{
-		Gtk.StockItem item = Gtk.StockItem.Zero;
-		if (Gtk.StockManager.Lookup (stock_id, ref item)) {
-			SignalFuncHelper helper = new SignalFuncHelper (e);
-
-			// FIXME current gtk-sharp bindings don't have a null_ok flag on the 
-			// widget parameter so it is impossible to make a toggle button in toolbar.
-			Gtk.Widget w;
+		try {
+			return theme.LoadIcon (icon_name, size, flags);
+		} catch {
 			try {
-				w =  toolbar.AppendElement (Gtk.ToolbarChildType.Togglebutton, 
-							    null,
-							    item.Label.Replace ("_", null),
-							    null, null, 
-							    new Gtk.Image (item.StockId, Gtk.IconSize.LargeToolbar), 
-							    new Gtk.SignalFunc (helper.Func));
+				return theme.LoadIcon ("gtk-missing-image", size, flags);
 			} catch {
-				w =  toolbar.AppendItem (item.Label.Replace ("_", null),
-							 null, null, 
-							 new Gtk.Image (item.StockId, Gtk.IconSize.LargeToolbar), 
-							 new Gtk.SignalFunc (helper.Func));
+				return null;
 			}
-
-			helper.Sender = w;
-			return w;
-		}
-		return null;
+		}	
 	}
 }
 
 public class SizeUtil {
 	public static string ToHumanReadable (long size)
 	{
-		string tmp_str = "";
+		string tmp_str = String.Empty;
 		float tmp_size = size;
 		int k = 0;
 		string[] size_abr = {"bytes", "kB", "MB", "GB", "TB" };
@@ -428,7 +407,7 @@ public class SizeUtil {
 	}
 }
 
-class GnomeUtil {
+public class GnomeUtil {
 	Gtk.Window window;
 	string url;
 	
@@ -447,8 +426,8 @@ class GnomeUtil {
 	       		HigMessageDialog md = new HigMessageDialog (window, Gtk.DialogFlags.DestroyWithParent, 
 	       			Gtk.MessageType.Error, Gtk.ButtonsType.Ok, 
 	       			Mono.Unix.Catalog.GetString ("There was an error invoking the external handler"),
-	       			String.Format (Mono.Unix.Catalog.GetString ("Received error:\n\"{0}\"\n"), 
-	       			ge.Message));
+	       			String.Format (Mono.Unix.Catalog.GetString ("Received error:{1}\"{0}\"{1}"), 
+	       			ge.Message, Environment.NewLine));
 
 	       		md.Run ();
 	       		md.Destroy ();
